@@ -8,7 +8,6 @@
 #include "dballe/core/csv.h"
 #include "dballe/core/json.h"
 #include "dballe/core/match-wreport.h"
-#include "dballe/msg/aof_codec.h"
 #include "dballe/cmdline/cmdline.h"
 #include <cstring>
 #include <cstdlib>
@@ -52,13 +51,13 @@ Item::~Item()
     if (rmsg) delete rmsg;
 }
 
-void Item::set_msgs(Messages* new_msgs)
+void Item::set_msgs(std::vector<std::shared_ptr<dballe::Message>>* new_msgs)
 {
     if (msgs) delete msgs;
     msgs = new_msgs;
 }
 
-void Item::decode(msg::Importer& imp, bool print_errors)
+void Item::decode(Importer& imp, bool print_errors)
 {
     if (!rmsg) return;
 
@@ -77,7 +76,7 @@ void Item::decode(msg::Importer& imp, bool print_errors)
     // First step: decode raw message to bulletin
     switch (rmsg->encoding)
     {
-        case File::BUFR:
+        case Encoding::BUFR:
             try {
                 bulletin = BufrBulletin::decode(rmsg->data, rmsg->pathname.c_str(), rmsg->offset).release();
             } catch (error& e) {
@@ -86,7 +85,7 @@ void Item::decode(msg::Importer& imp, bool print_errors)
                 bulletin = 0;
             }
             break;
-        case File::CREX:
+        case Encoding::CREX:
             try {
                 bulletin = CrexBulletin::decode(rmsg->data, rmsg->pathname.c_str(), rmsg->offset).release();
             } catch (error& e) {
@@ -95,19 +94,16 @@ void Item::decode(msg::Importer& imp, bool print_errors)
                 bulletin = 0;
             }
             break;
-        case File::AOF:
-            // Nothing to do for AOF
-            break;
     }
 
     // Second step: decode to msgs
     switch (rmsg->encoding)
     {
-        case File::BUFR:
-        case File::CREX:
+        case Encoding::BUFR:
+        case Encoding::CREX:
             if (bulletin)
             {
-                msgs = new Messages;
+                msgs = new std::vector<std::shared_ptr<dballe::Message>>;
                 try {
                     *msgs = imp.from_bulletin(*bulletin);
                 } catch (error& e) {
@@ -117,19 +113,13 @@ void Item::decode(msg::Importer& imp, bool print_errors)
                 }
             }
             break;
-        case File::AOF:
-            msgs = new Messages;
-            try {
-                *msgs = imp.from_binary(*rmsg);
-            } catch (error& e) {
-                if (print_errors) print_parse_error(*rmsg, e);
-                delete msgs;
-                msgs = 0;
-            }
-            break;
     }
 }
 
+void Item::processing_failed(std::exception& e) const
+{
+    throw ProcessingException(rmsg ? rmsg->pathname : "(unknown)", idx, e);
+}
 
 void IndexMatcher::parse(const std::string& str)
 {
@@ -208,7 +198,7 @@ bool Filter::match_index(int idx) const
     return imatcher.match(idx);
 }
 
-bool Filter::match_common(const BinaryMessage&, const Messages* msgs) const
+bool Filter::match_common(const BinaryMessage&, const std::vector<std::shared_ptr<dballe::Message>>* msgs) const
 {
     if (msgs == NULL && parsable)
         return false;
@@ -217,7 +207,7 @@ bool Filter::match_common(const BinaryMessage&, const Messages* msgs) const
     return true;
 }
 
-bool Filter::match_bufrex(const BinaryMessage& rmsg, const Bulletin* rm, const Messages* msgs) const
+bool Filter::match_bufrex(const BinaryMessage& rmsg, const Bulletin* rm, const std::vector<std::shared_ptr<dballe::Message>>* msgs) const
 {
     if (!match_common(rmsg, msgs))
         return false;
@@ -245,14 +235,14 @@ bool Filter::match_bufrex(const BinaryMessage& rmsg, const Bulletin* rm, const M
     return true;
 }
 
-bool Filter::match_bufr(const BinaryMessage& rmsg, const Bulletin* rm, const Messages* msgs) const
+bool Filter::match_bufr(const BinaryMessage& rmsg, const Bulletin* rm, const std::vector<std::shared_ptr<dballe::Message>>* msgs) const
 {
     if (!match_bufrex(rmsg, rm, msgs))
         return false;
     return true;
 }
 
-bool Filter::match_crex(const BinaryMessage& rmsg, const Bulletin* rm, const Messages* msgs) const
+bool Filter::match_crex(const BinaryMessage& rmsg, const Bulletin* rm, const std::vector<std::shared_ptr<dballe::Message>>* msgs) const
 {
     if (!match_bufrex(rmsg, rm, msgs))
         return false;
@@ -272,30 +262,9 @@ bool Filter::match_crex(const BinaryMessage& rmsg, const Bulletin* rm, const Mes
 #endif
 }
 
-bool Filter::match_aof(const BinaryMessage& rmsg, const Messages* msgs) const
+bool Filter::match_msgs(const std::vector<std::shared_ptr<dballe::Message>>& msgs) const
 {
-    int category, subcategory;
-    msg::AOFImporter::get_category(rmsg, &category, &subcategory);
-
-    if (!match_common(rmsg, msgs))
-        return false;
-
-    if (this->category != -1)
-        if (this->category != category)
-            return false;
-
-    if (this->subcategory != -1)
-        if (this->subcategory != subcategory)
-            return false;
-
-    if (msgs) return match_msgs(*msgs);
-
-    return true;
-}
-
-bool Filter::match_msgs(const Messages& msgs) const
-{
-    if (matcher && matcher->match(MatchedMessages(msgs)) != matcher::MATCH_YES)
+    if (matcher && matcher->match(impl::MatchedMessages(msgs)) != matcher::MATCH_YES)
         return false;
 
     return true;
@@ -307,9 +276,8 @@ bool Filter::match_item(const Item& item) const
     {
         switch (item.rmsg->encoding)
         {
-            case File::BUFR: return match_bufr(*item.rmsg, item.bulletin, item.msgs);
-            case File::CREX: return match_crex(*item.rmsg, item.bulletin, item.msgs);
-            case File::AOF: return match_aof(*item.rmsg, item.msgs);
+            case Encoding::BUFR: return match_bufr(*item.rmsg, item.bulletin, item.msgs);
+            case Encoding::CREX: return match_crex(*item.rmsg, item.bulletin, item.msgs);
             default: return false;
         }
     } else if (item.msgs)
@@ -319,10 +287,15 @@ bool Filter::match_item(const Item& item) const
 }
 
 Reader::Reader(const ReaderOptions& opts)
-    : input_type(opts.input_type), fail_file_name(opts.fail_file_name), filter(opts), verbose(false)
+    : input_type(opts.input_type), fail_file_name(opts.fail_file_name), filter(opts)
 {
 }
 
+bool Reader::has_fail_file() const
+{
+    return fail_file_name != nullptr;
+}
+
 void Reader::read_csv(const std::list<std::string>& fnames, Action& action)
 {
     // This cannot be implemented in dballe::File at the moment, since
@@ -348,7 +321,7 @@ void Reader::read_csv(const std::list<std::string>& fnames, Action& action)
         while (true)
         {
             // Read input message
-            unique_ptr<Msg> msg(new Msg);
+            unique_ptr<impl::Message> msg(new impl::Message);
             if (!msg->from_csv(*csvin))
                 break;
 
@@ -358,8 +331,8 @@ void Reader::read_csv(const std::list<std::string>& fnames, Action& action)
                 continue;
 
             // We want it: move it to the item
-            unique_ptr<Messages> msgs(new Messages);
-            msgs->append(move(msg));
+            unique_ptr<impl::Messages> msgs(new impl::Messages);
+            msgs->emplace_back(move(msg));
             item.set_msgs(msgs.release());
 
             if (!filter.match_item(item))
@@ -372,12 +345,14 @@ void Reader::read_csv(const std::list<std::string>& fnames, Action& action)
 
 void Reader::read_json(const std::list<std::string>& fnames, Action& action)
 {
+    using core::JSONParseException;
+
     struct JSONMsgReader : public core::JSONReader {
         std::istream* in;
         bool close_on_exit;
 
-        Msg msg;
-        std::unique_ptr<msg::Context> ctx;
+        impl::Message msg;
+        std::unique_ptr<impl::msg::Context> ctx;
         std::unique_ptr<wreport::Var> var;
         std::unique_ptr<wreport::Var> attr;
 
@@ -420,13 +395,15 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
         JSONMsgReader(std::istream& in) : in(&in), close_on_exit(false) {}
         JSONMsgReader(const std::string& name) : close_on_exit(true) {
             in = new ifstream(name);
+            if (not in->good())
+                throw runtime_error(strerror(errno));
         }
         ~JSONMsgReader() {
             if (close_on_exit)
                 delete in;
         }
 
-        void parse_msgs(std::function<void(const Msg&)> cb) {
+        void parse_msgs(std::function<void(const impl::Message&)> cb) {
             if (in) {
                 while (!in->eof())
                 {
@@ -439,12 +416,12 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                 }
             }
             if (not state.empty())
-                throw std::runtime_error("Incomplete JSON");
+                throw JSONParseException("Incomplete JSON");
         }
 
         void throw_error_if_empty_state() {
             if (state.empty())
-                throw std::runtime_error("Invalid JSON value");
+                throw JSONParseException("Invalid JSON value");
         }
 
         virtual void on_start_list() {
@@ -465,7 +442,7 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                     state.push(MSG_DATA_LIST_ITEM_TRANGE_LIST);
                     state.push(MSG_DATA_LIST_ITEM_TRANGE_LIST_PIND);
                     break;
-                default: throw std::runtime_error("Invalid JSON value start_list");
+                default: throw JSONParseException("Invalid JSON value start_list");
             }
         }
         virtual void on_end_list() {
@@ -475,7 +452,7 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                 case MSG_DATA_LIST: state.pop(); break;
                 case MSG_DATA_LIST_ITEM_LEVEL_LIST: state.pop(); break;
                 case MSG_DATA_LIST_ITEM_TRANGE_LIST: state.pop(); break;
-                default: throw std::runtime_error("Invalid JSON value end_list");
+                default: throw JSONParseException("Invalid JSON value end_list");
             }
         }
         virtual void on_start_mapping() {
@@ -490,7 +467,7 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                 switch (s) {
                     case MSG_DATA_LIST:
                         state.push(MSG_DATA_LIST_ITEM);
-                        ctx.reset(new msg::Context(Level(), Trange()));
+                        ctx.reset(new impl::msg::Context(Level(), Trange()));
                         break;
                     case MSG_DATA_LIST_ITEM_VARS_MAPPING_VAR:
                         state.pop();
@@ -504,7 +481,7 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                         state.pop();
                         state.push(MSG_DATA_LIST_ITEM_VARS_MAPPING_ATTR_MAPPING);
                         break;
-                    default: throw std::runtime_error("Invalid JSON value start_mapping");
+                    default: throw JSONParseException("Invalid JSON value start_mapping");
                 }
             }
         }
@@ -524,9 +501,13 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                     // of "lon", "lat", "ident", "network".
                     // Then, context overwrite is allowed.
                     // msg.add_context(std::move(ctx));
-                    msg::Context& ctx2 = msg.obtain_context(ctx->level, ctx->trange);
-                    for (const auto& ci: ctx->data)
-                        ctx2.set(*ci);
+                    if (ctx->level.is_missing() && ctx->trange.is_missing())
+                    {
+                        msg.station_data.merge(ctx->values);
+                    } else {
+                        impl::msg::Context& ctx2 = msg.obtain_context(ctx->level, ctx->trange);
+                        ctx2.values.merge(ctx->values);
+                    }
                     state.pop();
                     break;
                 }
@@ -535,7 +516,7 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_VAR_MAPPING:
                     state.pop();
                     break;
-                default: throw std::runtime_error("Invalid JSON value end_mapping");
+                default: throw JSONParseException("Invalid JSON value end_mapping");
             }
         }
         virtual void on_add_null() {
@@ -580,15 +561,16 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_VAR_KEY:
                     var->unset();
-                    ctx->set(*var);
+                    ctx->values.set(*var);
                     state.pop();
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_ATTR_MAPPING_VAR_KEY:
                     state.pop();
                     attr->set(MISSING_INT);
                     var->seta(*attr);
+                    ctx->values.set(*var);
                     break;
-                default: throw std::runtime_error("Invalid JSON value add_null");
+                default: throw JSONParseException("Invalid JSON value add_null");
             }
         }
         virtual void on_add_bool(bool val) {
@@ -597,15 +579,16 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
             switch (s) {
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_VAR_KEY:
                     var->set(val);
-                    ctx->set(*var);
+                    ctx->values.set(*var);
                     state.pop();
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_ATTR_MAPPING_VAR_KEY:
                     state.pop();
                     attr->set(val);
                     var->seta(*attr);
+                    ctx->values.set(*var);
                     break;
-                default: throw std::runtime_error("Invalid JSON value add_bool");
+                default: throw JSONParseException("Invalid JSON value add_bool");
             }
         }
         virtual void on_add_int(int val) {
@@ -654,16 +637,19 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                     state.pop();
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_VAR_KEY:
-                    var->set(val);
-                    ctx->set(*var);
+                    // Var::seti on decimal vars is considered as the value
+                    // with the scale already applied
+                    var->setf(to_string(val).c_str());
+                    ctx->values.set(*var);
                     state.pop();
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_ATTR_MAPPING_VAR_KEY:
                     state.pop();
                     attr->set(val);
                     var->seta(*attr);
+                    ctx->values.set(*var);
                     break;
-                default: throw std::runtime_error("Invalid JSON value add_int");
+                default: throw JSONParseException("Invalid JSON value add_int");
             }
         }
         virtual void on_add_double(double val) {
@@ -672,15 +658,16 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
             switch (s) {
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_VAR_KEY:
                     var->set(val);
-                    ctx->set(*var);
+                    ctx->values.set(*var);
                     state.pop();
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_ATTR_MAPPING_VAR_KEY:
                     state.pop();
                     attr->set(val);
                     var->seta(*attr);
+                    ctx->values.set(*var);
                     break;
-                default: throw std::runtime_error("Invalid JSON value add_double");
+                default: throw JSONParseException("Invalid JSON value add_double");
             }
         }
         virtual void on_add_string(const std::string& val) {
@@ -703,7 +690,7 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                     else if (val == "data")
                         state.push(MSG_DATA_KEY);
                     else
-                        throw std::runtime_error("Invalid JSON value");
+                        throw JSONParseException("Invalid JSON value");
                     break;
                 case MSG_IDENT_KEY:
                     msg.set_ident(val.c_str());
@@ -711,7 +698,7 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                     break;
                 case MSG_VERSION_KEY:
                     if (strcmp(val.c_str(), DBALLE_JSON_VERSION) != 0)
-                        throw std::runtime_error("Invalid JSON version " + val);
+                        throw JSONParseException("Invalid JSON version " + val);
                     state.pop();
                     break;
                 case MSG_NETWORK_KEY:
@@ -730,11 +717,11 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                     else if (val == "timerange")
                         state.push(MSG_DATA_LIST_ITEM_TRANGE_KEY);
                     else
-                        throw std::runtime_error("Invalid JSON value");
+                        throw JSONParseException("Invalid JSON value");
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING:
                     state.push(MSG_DATA_LIST_ITEM_VARS_MAPPING_VAR);
-                    var.reset(new Var(dballe::var(val)));
+                    var = newvar(val);
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_VAR_MAPPING:
                     if (val == "v")
@@ -742,23 +729,24 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
                     else if (val == "a")
                         state.push(MSG_DATA_LIST_ITEM_VARS_MAPPING_ATTR_KEY);
                     else
-                        throw std::runtime_error("Invalid JSON value");
+                        throw JSONParseException("Invalid JSON value");
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_VAR_KEY:
                     var->set(val);
-                    ctx->set(*var);
+                    ctx->values.set(*var);
                     state.pop();
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_ATTR_MAPPING:
                     state.push(MSG_DATA_LIST_ITEM_VARS_MAPPING_ATTR_MAPPING_VAR_KEY);
-                    attr.reset(new Var(dballe::var(val)));
+                    attr = newvar(val);
                     break;
                 case MSG_DATA_LIST_ITEM_VARS_MAPPING_ATTR_MAPPING_VAR_KEY:
                     state.pop();
                     attr->set(val);
                     var->seta(*attr);
+                    ctx->values.set(*var);
                     break;
-                default: throw std::runtime_error("Invalid JSON value add_string");
+                default: throw JSONParseException("Invalid JSON value add_string");
             }
         }
     };
@@ -766,27 +754,36 @@ void Reader::read_json(const std::list<std::string>& fnames, Action& action)
     std::unique_ptr<JSONMsgReader> jsonreader;
 
     list<string>::const_iterator name = fnames.begin();
-    do {
-        if (name != fnames.end()) {
-            jsonreader.reset(new JSONMsgReader(*name));
-            ++name;
-        } else {
-            jsonreader.reset(new JSONMsgReader(cin));
-        }
-        jsonreader->parse_msgs([&](const Msg& msg) {
-            ++item.idx;
-            if (!filter.match_index(item.idx))
-                return;
-            unique_ptr<Messages> msgs(new Messages);
-            msgs->append(msg);
-            item.set_msgs(msgs.release());
+    try {
+        do {
+            if (name != fnames.end()) {
+                jsonreader.reset(new JSONMsgReader(*name));
+                ++name;
+            } else {
+                jsonreader.reset(new JSONMsgReader(cin));
+            }
+            jsonreader->parse_msgs([&](const impl::Message& msg) {
+                ++item.idx;
+                if (!filter.match_index(item.idx))
+                    return;
+                unique_ptr<impl::Messages> msgs(new impl::Messages);
+                msgs->emplace_back(make_shared<impl::Message>(msg));
+                item.set_msgs(msgs.release());
 
-            if (!filter.match_item(item))
-                return;
+                if (!filter.match_item(item))
+                    return;
 
-            action(item);
-        });
-    } while (name != fnames.end());
+                action(item);
+            });
+        } while (name != fnames.end());
+    } catch (const JSONParseException& e) {
+        // If name points to begin(), then it's the standard input, because
+        // after parsing a file the iterator is incremented.
+        const std::string f = ( name != fnames.begin() ? *(--name) : "stdin" );
+        throw JSONParseException("Error while parsing JSON from " + f +
+                                 " at char " + std::to_string(jsonreader->in->tellg()) +
+                                 ": " + e.what());
+    }
 }
 
 void Reader::read_file(const std::list<std::string>& fnames, Action& action)
@@ -809,7 +806,7 @@ void Reader::read_file(const std::list<std::string>& fnames, Action& action)
                 file = File::create(stdin, false, "standard input");
             }
         } else {
-            File::Encoding intype = string_to_encoding(input_type.c_str());
+            Encoding intype = string_to_encoding(input_type.c_str());
             if (name != fnames.end())
             {
                 file = File::create(intype, *name, "r");
@@ -820,7 +817,7 @@ void Reader::read_file(const std::list<std::string>& fnames, Action& action)
         }
 
 
-        std::unique_ptr<msg::Importer> imp = msg::Importer::create(file->encoding(), import_opts);
+        std::unique_ptr<Importer> imp = Importer::create(file->encoding(), import_opts);
         while (BinaryMessage bm = file->read())
         {
             Item item;
@@ -835,7 +832,15 @@ void Reader::read_file(const std::list<std::string>& fnames, Action& action)
                 if (!filter.match_index(item.idx))
                     continue;
 
-                item.decode(*imp, print_errors);
+                try {
+                    item.decode(*imp, print_errors);
+                } catch (std::exception& e) {
+                    // Convert decode errors into ProcessingException, to skip
+                    // this item if it fails to decode. We can safely skip,
+                    // because if file->read() returned successfully the next
+                    // read should properly start at the next item
+                    item.processing_failed(e);
+                }
 
                 //process_input(*file, rmsg, grepdata, action);
 
@@ -862,6 +867,10 @@ void Reader::read_file(const std::list<std::string>& fnames, Action& action)
                     fail_file = File::create(file->encoding(), fail_file_name, "ab");
                 fail_file->write(item.rmsg->data);
             }
+            if (processed)
+                ++count_successes;
+            else
+                ++count_failures;
         }
     } while (name != fnames.end());
 }
