@@ -1,7 +1,6 @@
 #include "query.h"
 #include "var.h"
 #include "json.h"
-#include "record.h"
 #include <sstream>
 #include <cmath>
 #include <cstring>
@@ -12,6 +11,13 @@ using namespace std;
 namespace dballe {
 namespace core {
 
+void Query::validate()
+{
+    lonrange.set(lonrange);
+    dtrange.min.set_lower_bound();
+    dtrange.max.set_upper_bound();
+}
+
 std::unique_ptr<dballe::Query> Query::clone() const
 {
     return unique_ptr<Query>(new Query(*this));
@@ -41,14 +47,14 @@ unsigned Query::get_modifiers() const
 void Query::clear()
 {
     ana_id = MISSING_INT;
-    prio_min = MISSING_INT;
-    prio_max = MISSING_INT;
-    rep_memo.clear();
+    priomin = MISSING_INT;
+    priomax = MISSING_INT;
+    report.clear();
     mobile = MISSING_INT;
     ident.clear();
     latrange = LatRange();
     lonrange = LonRange();
-    datetime = DatetimeRange();
+    dtrange = DatetimeRange();
     level = Level();
     trange = Trange();
     varcodes.clear();
@@ -61,110 +67,42 @@ void Query::clear()
     station = MISSING_INT;
 }
 
-void Query::set_from_record(const dballe::Record& rec)
+void Query::set_from_string(const char* str)
 {
-    const auto& r = core::Record::downcast(rec);
+    // Split the input as name=val
+    const char* s = strchr(str, '=');
 
-    want_missing = 0;
-    // Ana ID
-    ana_id = rec.enq("ana_id", MISSING_INT);
-    // Priority
-    int i = rec.enq("priority", MISSING_INT);
-    if (i != MISSING_INT)
-        prio_min = prio_max = i;
+    if (!s) error_consistency::throwf("there should be an = between the name and the value in '%s'", str);
+
+    string key(str, s - str);
+    const char* val = s + 1;
+    if (strcmp(val, "-") == 0)
+        unset(key.data(), key.size());
     else
-    {
-        prio_min = rec.enq("priomin", MISSING_INT);
-        prio_max = rec.enq("priomax", MISSING_INT);
-    }
-    // Network
-    rep_memo = rec.enq("rep_memo", "");
-    // Mobile
-    mobile = rec.enq("mobile", MISSING_INT);
-    // Ident
-    if (const Var* var = rec.get("ident"))
-        ident = var->enqc();
-    else
-        ident.clear();
-    // Latitude
-    i = rec.enq("lat", MISSING_INT);
-    if (i != MISSING_INT)
-        latrange.set(i, i);
-    else
-    {
-        int imin = rec.enq("latmin", MISSING_INT);
-        int imax = rec.enq("latmax", MISSING_INT);
-        latrange.set(
-                imin == MISSING_INT ? LatRange::IMIN : imin,
-                imax == MISSING_INT ? LatRange::IMAX : imax);
-    }
-    // Longitude
-    i = rec.enq("lon", MISSING_INT);
-    if (i != MISSING_INT)
-        lonrange.set(i, i);
-    else
-        lonrange.set(rec.enq("lonmin", MISSING_INT), rec.enq("lonmax", MISSING_INT));
-    // Datetime
-    // fetch all values involved in the computation
-    int ye = rec.enq("year", MISSING_INT);
-    int mo = rec.enq("month", MISSING_INT);
-    int da = rec.enq("day", MISSING_INT);
-    int ho = rec.enq("hour", MISSING_INT);
-    int mi = rec.enq("min", MISSING_INT);
-    int se = rec.enq("sec", MISSING_INT);
-    int yemin = rec.enq("yearmin", MISSING_INT);
-    int momin = rec.enq("monthmin", MISSING_INT);
-    int damin = rec.enq("daymin", MISSING_INT);
-    int homin = rec.enq("hourmin", MISSING_INT);
-    int mimin = rec.enq("minumin", MISSING_INT);
-    int semin = rec.enq("secmin", MISSING_INT);
-    int yemax = rec.enq("yearmax", MISSING_INT);
-    int momax = rec.enq("monthmax", MISSING_INT);
-    int damax = rec.enq("daymax", MISSING_INT);
-    int homax = rec.enq("hourmax", MISSING_INT);
-    int mimax = rec.enq("minumax", MISSING_INT);
-    int semax = rec.enq("secmax", MISSING_INT);
-    // give absolute values priority over ranges
-    if (ye != MISSING_INT) yemin = yemax = ye;
-    if (mo != MISSING_INT) momin = momax = mo;
-    if (da != MISSING_INT) damin = damax = da;
-    if (ho != MISSING_INT) homin = homax = ho;
-    if (mi != MISSING_INT) mimin = mimax = mi;
-    if (se != MISSING_INT) semin = semax = se;
-    datetime = DatetimeRange(yemin, momin, damin, homin, mimin, semin, yemax, momax, damax, homax, mimax, semax);
-    // Level
-    level = r.get_level();
-    // Trange
-    trange = r.get_trange();
-    // Varcodes
-    varcodes.clear();
-    if (const Var* var = rec.get("var"))
-        varcodes.insert(resolve_varcode(var->enq("")));
-    else if (const Var* var = rec.get("varlist"))
-        resolve_varlist(var->enq(""), varcodes);
-    // Query
-    query = rec.enq("query", "");
-    // Filters
-    ana_filter = rec.enq("ana_filter", "");
-    data_filter = rec.enq("data_filter", "");
-    attr_filter = rec.enq("attr_filter", "");
-    // Limit
-    limit = rec.enq("limit", MISSING_INT);
-    // WMO block/station
-    block = rec.enq("block", MISSING_INT);
-    station = rec.enq("station", MISSING_INT);
+        setf(key.data(), key.size(), val);
 }
 
 void Query::set_from_test_string(const std::string& s)
 {
+    clear();
     if (s.empty())
-        clear();
-    else
+        return;
+
+    size_t cur = 0;
+    while (true)
     {
-        core::Record rec;
-        rec.set_from_test_string(s);
-        set_from_record(rec);
+        size_t next = s.find(", ", cur);
+        if (next == string::npos)
+        {
+            set_from_string(s.substr(cur).c_str());
+            break;
+        } else {
+            set_from_string(s.substr(cur, next - cur).c_str());
+            cur = next + 2;
+        }
     }
+
+    validate();
 }
 
 namespace {
@@ -235,13 +173,13 @@ bool Query::is_subquery(const dballe::Query& other_gen) const
     const Query& other = downcast(other_gen);
 
     if (removed_or_changed(ana_id, other.ana_id)) return false;
-    if (!is_subrange(prio_min, prio_max, other.prio_min, other.prio_max)) return false;
-    if (removed_or_changed(rep_memo, other.rep_memo)) return false;
+    if (!is_subrange(priomin, priomax, other.priomin, other.priomax)) return false;
+    if (removed_or_changed(report, other.report)) return false;
     if (removed_or_changed(mobile, other.mobile)) return false;
     if (removed_or_changed(ident, other.ident)) return false;
     if (!other.latrange.contains(latrange)) return false;
     if (!other.lonrange.contains(lonrange)) return false;
-    if (!other.datetime.contains(datetime)) return false;
+    if (!other.dtrange.contains(dtrange)) return false;
     if (removed_or_changed(level.ltype1, other.level.ltype1)) return false;
     if (removed_or_changed(level.l1, other.level.l1)) return false;
     if (removed_or_changed(level.ltype2, other.level.ltype2)) return false;
@@ -258,7 +196,7 @@ bool Query::is_subquery(const dballe::Query& other_gen) const
     if (mods != omods)
     {
         // The only relevant bits is query=best, all the rest we can safely ignore
-        if ((mods & DBA_DB_MODIFIER_BEST) != (omods & DBA_DB_MODIFIER_BEST)) return false;
+        if (!(mods & DBA_DB_MODIFIER_BEST) && (omods & DBA_DB_MODIFIER_BEST)) return false;
     }
     if (removed_or_changed(ana_filter, other.ana_filter)) return false;
     if (removed_or_changed(data_filter, other.data_filter)) return false;
@@ -272,148 +210,6 @@ bool Query::is_subquery(const dballe::Query& other_gen) const
 
 namespace {
 
-struct VarGen
-{
-    std::function<void(const char*, Var&&)>& dest;
-
-    VarGen(std::function<void(const char*, Var&&)>& dest)
-        : dest(dest) {}
-
-    template<typename T>
-    void gen(dba_keyword key, const T& val)
-    {
-        Varinfo info = Record::keyword_info(key);
-        dest(Record::keyword_name(key), Var(info, val));
-    };
-
-    void gen(dba_keyword key, const std::string& val)
-    {
-        Varinfo info = Record::keyword_info(key);
-        dest(Record::keyword_name(key), Var(info, val.c_str()));
-    };
-
-    void gen(dba_keyword key, const Ident& val)
-    {
-        Varinfo info = Record::keyword_info(key);
-        dest(Record::keyword_name(key), Var(info, val.get()));
-    };
-};
-
-}
-
-void Query::foreach_key(std::function<void(const char*, Var&&)> dest) const
-{
-    VarGen vargen(dest);
-
-    if (ana_id != MISSING_INT) vargen.gen(DBA_KEY_ANA_ID, ana_id);
-    if (prio_min != prio_max)
-    {
-        if (prio_min != MISSING_INT)
-            vargen.gen(DBA_KEY_PRIORITY, prio_min);
-    } else {
-        if (prio_min != MISSING_INT)
-            vargen.gen(DBA_KEY_PRIOMIN, prio_min);
-        if (prio_max != MISSING_INT)
-            vargen.gen(DBA_KEY_PRIOMAX, prio_max);
-    }
-    if (!rep_memo.empty())
-        vargen.gen(DBA_KEY_REP_MEMO, rep_memo);
-    if (mobile != MISSING_INT)
-        vargen.gen(DBA_KEY_MOBILE, mobile);
-    if (!ident.is_missing())
-        vargen.gen(DBA_KEY_IDENT, ident);
-    if (!latrange.is_missing())
-    {
-        if (latrange.imin == latrange.imax)
-            vargen.gen(DBA_KEY_LAT, latrange.imin);
-        else
-        {
-            if (latrange.imin != LatRange::IMIN) vargen.gen(DBA_KEY_LATMIN, latrange.imin);
-            if (latrange.imax != LatRange::IMAX) vargen.gen(DBA_KEY_LATMAX, latrange.imax);
-        }
-    }
-    if (!lonrange.is_missing())
-    {
-        if (lonrange.imin == lonrange.imax)
-            vargen.gen(DBA_KEY_LON, lonrange.imin);
-        else
-        {
-            vargen.gen(DBA_KEY_LONMIN, lonrange.imin);
-            vargen.gen(DBA_KEY_LONMAX, lonrange.imax);
-        }
-    }
-    if (datetime.min == datetime.max)
-    {
-        if (!datetime.min.is_missing())
-        {
-            vargen.gen(DBA_KEY_YEAR, datetime.min.year);
-            vargen.gen(DBA_KEY_MONTH, datetime.min.month);
-            vargen.gen(DBA_KEY_DAY, datetime.min.day);
-            vargen.gen(DBA_KEY_HOUR, datetime.min.hour);
-            vargen.gen(DBA_KEY_MIN, datetime.min.minute);
-            vargen.gen(DBA_KEY_SEC, datetime.min.second);
-        }
-    } else {
-        if (!datetime.min.is_missing())
-        {
-            vargen.gen(DBA_KEY_YEARMIN, datetime.min.year);
-            vargen.gen(DBA_KEY_MONTHMIN, datetime.min.month);
-            vargen.gen(DBA_KEY_DAYMIN, datetime.min.day);
-            vargen.gen(DBA_KEY_HOURMIN, datetime.min.hour);
-            vargen.gen(DBA_KEY_MINUMIN, datetime.min.minute);
-            vargen.gen(DBA_KEY_SECMIN, datetime.min.second);
-        }
-        if (!datetime.max.is_missing())
-        {
-            vargen.gen(DBA_KEY_YEARMAX, datetime.max.year);
-            vargen.gen(DBA_KEY_MONTHMAX, datetime.max.month);
-            vargen.gen(DBA_KEY_DAYMAX, datetime.max.day);
-            vargen.gen(DBA_KEY_HOURMAX, datetime.max.hour);
-            vargen.gen(DBA_KEY_MINUMAX, datetime.max.minute);
-            vargen.gen(DBA_KEY_SECMAX, datetime.max.second);
-        }
-    }
-    if (level.ltype1 != MISSING_INT) vargen.gen(DBA_KEY_LEVELTYPE1, level.ltype1);
-    if (level.l1 != MISSING_INT) vargen.gen(DBA_KEY_L1, level.l1);
-    if (level.ltype2 != MISSING_INT) vargen.gen(DBA_KEY_LEVELTYPE2, level.ltype2);
-    if (level.l2 != MISSING_INT) vargen.gen(DBA_KEY_L2, level.l2);
-    if (trange.pind != MISSING_INT) vargen.gen(DBA_KEY_PINDICATOR, trange.pind);
-    if (trange.p1 != MISSING_INT) vargen.gen(DBA_KEY_P1, trange.p1);
-    if (trange.p2 != MISSING_INT) vargen.gen(DBA_KEY_P2, trange.p2);
-    switch (varcodes.size())
-    {
-         case 0:
-             break;
-         case 1:
-             vargen.gen(DBA_KEY_VAR, varcode_format(*varcodes.begin()));
-             break;
-         default: {
-             string codes;
-             for (const auto& code: varcodes)
-             {
-                 if (codes.empty())
-                     codes = varcode_format(code);
-                 else
-                 {
-                     codes += ",";
-                     codes += varcode_format(code);
-                 }
-             }
-             vargen.gen(DBA_KEY_VARLIST, codes);
-             break;
-         }
-    }
-    if (!query.empty()) vargen.gen(DBA_KEY_QUERY, query);
-    if (!ana_filter.empty()) vargen.gen(DBA_KEY_ANA_FILTER, ana_filter);
-    if (!data_filter.empty()) vargen.gen(DBA_KEY_DATA_FILTER, data_filter);
-    if (!attr_filter.empty()) vargen.gen(DBA_KEY_ATTR_FILTER, attr_filter);
-    if (limit != MISSING_INT) vargen.gen(DBA_KEY_LIMIT, limit);
-    if (block != MISSING_INT) dest("block", Var(varinfo(WR_VAR(0, 1, 1)), block));
-    if (station != MISSING_INT) dest("station", Var(varinfo(WR_VAR(0, 1, 2)), station));
-}
-
-namespace {
-
 struct Printer
 {
     const Query& q;
@@ -537,14 +333,14 @@ struct Printer
     void print()
     {
         print_int("ana_id", q.ana_id);
-        print_int("prio_min", q.prio_min);
-        print_int("prio_max", q.prio_max);
-        print_str("rep_memo", !q.rep_memo.empty(), q.rep_memo);
+        print_int("priomin", q.priomin);
+        print_int("priomax", q.priomax);
+        print_str("report", !q.report.empty(), q.report);
         print_int("mobile", q.mobile);
         print_ident(q.ident);
         print_latrange(q.latrange);
         print_lonrange(q.lonrange);
-        print_datetimerange(q.datetime);
+        print_datetimerange(q.dtrange);
         print_level("level", q.level);
         print_trange("trange", q.trange);
         print_varcodes("varcodes", q.varcodes);
@@ -570,9 +366,9 @@ void Query::print(FILE* out) const
 void Query::serialize(JSONWriter& out) const
 {
     if (ana_id != MISSING_INT) out.add("ana_id", ana_id);
-    if (prio_min != MISSING_INT) out.add("prio_min", prio_min);
-    if (prio_max != MISSING_INT) out.add("prio_max", prio_max);
-    if (!rep_memo.empty()) out.add("rep_memo", rep_memo);
+    if (priomin != MISSING_INT) out.add("prio_min", priomin);
+    if (priomax != MISSING_INT) out.add("prio_max", priomax);
+    if (!report.empty()) out.add("rep_memo", report);
     if (mobile != MISSING_INT) out.add("mobile", mobile);
     if (!ident.is_missing()) out.add("ident", ident.get());
     if (!latrange.is_missing())
@@ -585,12 +381,12 @@ void Query::serialize(JSONWriter& out) const
         out.add("lonmin", lonrange.imin);
         out.add("lonmax", lonrange.imax);
     }
-    if (datetime.min == datetime.max)
+    if (dtrange.min == dtrange.max)
     {
-        if (!datetime.min.is_missing()) out.add("datetime", datetime.min);
+        if (!dtrange.min.is_missing()) out.add("datetime", dtrange.min);
     } else {
-        if (!datetime.min.is_missing()) out.add("datetime_min", datetime.min);
-        if (!datetime.max.is_missing()) out.add("datetime_max", datetime.max);
+        if (!dtrange.min.is_missing()) out.add("datetime_min", dtrange.min);
+        if (!dtrange.max.is_missing()) out.add("datetime_max", dtrange.max);
     }
     if (!level.is_missing()) out.add("level", level);
     if (!trange.is_missing()) out.add("trange", trange);
@@ -608,15 +404,6 @@ void Query::serialize(JSONWriter& out) const
     if (station != MISSING_INT) out.add("station", station);
 }
 
-unsigned Query::parse_modifiers(const dballe::Record& rec)
-{
-    /* Decode query modifiers */
-    const Var* var = rec.get("query");
-    if (!var) return 0;
-    if (!var->isset()) return 0;
-    return parse_modifiers(var->enqc());
-}
-
 unsigned Query::parse_modifiers(const char* s)
 {
     unsigned modifiers = 0;
@@ -637,6 +424,12 @@ unsigned Query::parse_modifiers(const char* s)
                 else
                     got = 0;
                 break;
+            case 5:
+                if (strncmp(s, "attrs", 4) == 0)
+                    modifiers |= DBA_DB_MODIFIER_WITH_ATTRIBUTES;
+                else
+                    got = 0;
+                break;
             case 6:
                 /* "bigana": optimize with date first */
                 if (strncmp(s, "bigana", 6) == 0)
@@ -672,5 +465,63 @@ unsigned Query::parse_modifiers(const char* s)
     return modifiers;
 }
 
+Query Query::from_json(core::json::Stream& in)
+{
+    Query res;
+    in.parse_object([&](const std::string& key) {
+        if (key == "ana_id")
+            res.ana_id = in.parse_signed<int>();
+        else if (key == "prio_min")
+            res.priomin = in.parse_signed<int>();
+        else if (key == "prio_max")
+            res.priomax = in.parse_signed<int>();
+        else if (key == "rep_memo")
+            res.report = in.parse_string();
+        else if (key == "mobile")
+            res.mobile = in.parse_signed<int>();
+        else if (key == "ident")
+            res.ident = in.parse_string();
+        else if (key == "latmin")
+            res.latrange.imin = in.parse_signed<int>();
+        else if (key == "latmax")
+            res.latrange.imax = in.parse_signed<int>();
+        else if (key == "lonmin")
+            res.lonrange.imin = in.parse_signed<int>();
+        else if (key == "lonmax")
+            res.lonrange.imax = in.parse_signed<int>();
+        else if (key == "datetime")
+            res.dtrange.max = res.dtrange.min = in.parse_datetime();
+        else if (key == "datetime_min")
+            res.dtrange.min = in.parse_datetime();
+        else if (key == "datetime_max")
+            res.dtrange.max = in.parse_datetime();
+        else if (key == "level")
+            res.level = in.parse_level();
+        else if (key == "trange")
+            res.trange = in.parse_trange();
+        else if (key == "varcodes")
+        {
+            in.parse_array([&]{
+                res.varcodes.insert(in.parse_unsigned<wreport::Varcode>());
+            });
+        }
+        else if (key == "query")
+            res.query = in.parse_string();
+        else if (key == "ana_filter")
+            res.ana_filter = in.parse_string();
+        else if (key == "data_filter")
+            res.data_filter = in.parse_string();
+        else if (key == "attr_filter")
+            res.attr_filter = in.parse_string();
+        else if (key == "limit")
+            res.limit = in.parse_signed<int>();
+        else if (key == "block")
+            res.block = in.parse_signed<int>();
+        else if (key == "station")
+            res.station = in.parse_signed<int>();
+    });
+    return res;
+}
+
 }
 }
