@@ -1,11 +1,9 @@
 #include <wreport/tests.h>
 #include <dballe/file.h>
-#include <dballe/record.h>
+#include <dballe/values.h>
 #include <dballe/core/query.h>
-#include <dballe/core/values.h>
 #include <dballe/core/defs.h>
 #include <dballe/core/csv.h>
-#include <dballe/core/ostream.h>
 #include <cstdlib>
 #include <climits>
 #include <string>
@@ -52,9 +50,9 @@ static inline bool rnd(double prob)
 /// Return the pathname of a test file
 std::string datafile(const std::string& fname);
 
-std::unique_ptr<File> open_test_data(const char* filename, File::Encoding type);
+std::unique_ptr<File> open_test_data(const char* filename, Encoding type);
 
-BinaryMessage read_rawmsg(const char* filename, File::Encoding type);
+BinaryMessage read_rawmsg(const char* filename, Encoding type);
 
 class MemoryCSVWriter : public CSVWriter
 {
@@ -95,26 +93,7 @@ struct TestRecordVarsEqual
 };
 #endif
 
-struct ActualRecord : public wreport::tests::Actual<const dballe::Record&>
-{
-    ActualRecord(const dballe::Record& actual) : wreport::tests::Actual<const dballe::Record&>(actual) {}
-
-#if 0
-    TestRecordValEqual equals(const Record& expected, const char* name) { return TestRecordValEqual(this->actual, expected, name); }
-    TestRecordValEqual equals_with_missing_int(const Record& expected, const char* name)
-    {
-        return TestRecordValEqual(this->actual, expected, name, true);
-    }
-#endif
-    /// Check that actual and expected have the same vars
-    void vars_equal(const Record& expected) const { vars_equal(Values(expected)); }
-    /// Check that actual and expected have the same vars
-    void vars_equal(const Values& expected) const;
-};
-
-// Set a record from a ", "-separated string of assignments
-void set_record_from_string(Record& rec, const std::string& s);
-std::unique_ptr<Record> record_from_string(const std::string& s);
+// Set a query from a ", "-separated string of assignments
 std::unique_ptr<Query> query_from_string(const std::string& s);
 core::Query core_query_from_string(const std::string& s);
 
@@ -130,8 +109,6 @@ inline ActualMatcherResult actual_matcher_result(int actual) { return ActualMatc
 
 using wreport::tests::actual;
 
-inline dballe::tests::ActualRecord actual(const dballe::Record& actual) { return dballe::tests::ActualRecord(actual); }
-
 inline ActualCString actual(const dballe::Ident& ident) { return ActualCString(ident); }
 
 }
