@@ -1,5 +1,4 @@
 #include "levtr.h"
-#include "dballe/core/record.h"
 #include "dballe/msg/msg.h"
 
 using namespace std;
@@ -8,12 +7,27 @@ namespace dballe {
 namespace db {
 namespace v7 {
 
+LevTr::LevTr(v7::Transaction& tr) : tr(tr) {}
+
 LevTr::~LevTr() {}
 
-msg::Context* LevTr::to_msg(State& st, int id, Msg& msg)
+void LevTr::clear_cache()
 {
-    auto i = lookup_id(st, id);
-    msg::Context& res = msg.obtain_context(i->first.level, i->first.trange);
+    cache.clear();
+}
+
+const LevTrEntry& LevTr::lookup_cache(int id)
+{
+    const LevTrEntry* res = cache.find_entry(id);
+    if (!res)
+        wreport::error_notfound::throwf("LevTr with ID %d not found in cache", id);
+    return *res;
+}
+
+impl::msg::Context* LevTr::to_msg(Tracer<>& trc, int id, impl::Message& msg)
+{
+    auto i = lookup_id(trc, id);
+    impl::msg::Context& res = msg.obtain_context(i->level, i->trange);
     return &res;
 }
 
