@@ -25,23 +25,21 @@ protected:
      */
     dballe::sql::MySQLConnection& conn;
 
-    /// Lookup the ID of a station, returning true if it was found, false if not
-    bool maybe_get_id(const StationDesc& st, int* id) override;
-
     void _dump(std::function<void(int, int, const Coords& coords, const char* ident)> out) override;
 
 public:
-    MySQLStation(dballe::sql::MySQLConnection& conn);
+    MySQLStation(v7::Transaction& tr, dballe::sql::MySQLConnection& conn);
     ~MySQLStation();
     MySQLStation(const MySQLStation&) = delete;
     MySQLStation(const MySQLStation&&) = delete;
     MySQLStation& operator=(const MySQLStation&) = delete;
 
-    stations_t::iterator lookup_id(State& st, int id) override;
-    stations_t::iterator obtain_id(State& st, const StationDesc& desc) override;
-
-    void get_station_vars(int id_station, std::function<void(std::unique_ptr<wreport::Var>)> dest) override;
-    void add_station_vars(int id_station, Record& rec) override;
+    DBStation lookup(Tracer<>& trc, int id_station) override;
+    int maybe_get_id(Tracer<>& trc, const dballe::DBStation& st) override;
+    int insert_new(Tracer<>& trc, const dballe::DBStation& desc) override;
+    void get_station_vars(Tracer<>& trc, int id_station, std::function<void(std::unique_ptr<wreport::Var>)> dest) override;
+    void add_station_vars(Tracer<>& trc, int id_station, DBValues& values) override;
+    void run_station_query(Tracer<>& trc, const v7::StationQueryBuilder& qb, std::function<void(const dballe::DBStation&)>) override;
 };
 
 }
