@@ -3,18 +3,12 @@
 
 #include <dballe/db/db.h>
 #include <dballe/db/v7/levtr.h>
+#include <dballe/db/v7/fwd.h>
 #include <dballe/sql/fwd.h>
 #include <cstdio>
 #include <memory>
 
 namespace dballe {
-struct Record;
-struct Msg;
-
-namespace msg {
-struct Context;
-}
-
 namespace db {
 namespace v7 {
 namespace postgresql {
@@ -31,16 +25,15 @@ protected:
     void _dump(std::function<void(int, const Level&, const Trange&)> out) override;
 
 public:
-    PostgreSQLLevTr(dballe::sql::PostgreSQLConnection& conn);
+    PostgreSQLLevTr(v7::Transaction& tr, dballe::sql::PostgreSQLConnection& conn);
     PostgreSQLLevTr(const LevTr&) = delete;
     PostgreSQLLevTr(const LevTr&&) = delete;
     PostgreSQLLevTr& operator=(const PostgreSQLLevTr&) = delete;
     ~PostgreSQLLevTr();
 
-    void prefetch_ids(const std::set<int>& ids, std::function<void(int, const LevTrDesc&)> dest) override;
-    void prefetch_same_level(int id, std::function<void(int, const LevTrDesc&)> dest) override;
-    levtrs_t::iterator lookup_id(State& st, int id) override;
-    levtrs_t::iterator obtain_id(State& state, const LevTrDesc& desc) override;
+    void prefetch_ids(Tracer<>& trc, const std::set<int>& ids) override;
+    const LevTrEntry* lookup_id(Tracer<>& trc, int id) override;
+    int obtain_id(Tracer<>& trc, const LevTrEntry& desc) override;
 };
 
 
