@@ -1,4 +1,5 @@
 #include "qbuilder.h"
+#include "transaction.h"
 #include "dballe/core/defs.h"
 #include "dballe/core/aliases.h"
 #include "dballe/core/query.h"
@@ -176,17 +177,6 @@ struct Constraints
     Constraints(const core::Query& query, const char* tbl, Querybuf& q)
         : query(query), tbl(tbl), q(q), found(false) {}
 
-#if 0
-    void add_int(dba_keyword key, const char* sql)
-    {
-        const Var* var = rec.key_peek(key);
-        if (!var || !var->isset()) return;
-        //TRACE("found %s: adding %s. val is %d\n", info(key)->desc, sql, *out);
-        q.append_listf(sql, tbl, var->enqi());
-        found = true;
-    }
-#endif
-
     void add_lat()
     {
         if (query.latrange.is_missing()) return;
@@ -232,14 +222,14 @@ struct Constraints
     }
 };
 
-QueryBuilder::QueryBuilder(DB& db, const core::Query& query, unsigned int modifiers, bool query_station_vars)
-    : conn(*db.conn), db(db), query(query), sql_query(2048), sql_from(1024), sql_where(1024),
+QueryBuilder::QueryBuilder(std::shared_ptr<v7::Transaction> tr, const core::Query& query, unsigned int modifiers, bool query_station_vars)
+    : conn(*tr->db->conn), tr(tr), query(query), sql_query(2048), sql_from(1024), sql_where(1024),
       modifiers(modifiers), query_station_vars(query_station_vars)
 {
 }
 
-DataQueryBuilder::DataQueryBuilder(DB& db, const core::Query& query, unsigned int modifiers, bool query_station_vars, bool query_attrs)
-    : QueryBuilder(db, query, modifiers, query_station_vars), query_attrs(query_attrs)
+DataQueryBuilder::DataQueryBuilder(std::shared_ptr<v7::Transaction> tr, const core::Query& query, unsigned int modifiers, bool query_station_vars)
+    : QueryBuilder(tr, query, modifiers, query_station_vars), query_attrs(modifiers & DBA_DB_MODIFIER_WITH_ATTRIBUTES)
 {
 }
 
@@ -321,16 +311,16 @@ bool StationQueryBuilder::build_where()
             break;
     }
 
-    if (!query.rep_memo.empty())
+    if (!query.report.empty())
     {
-        int src_val = db.repinfo().get_id(query.rep_memo.c_str());
+        int src_val = tr->repinfo().get_id(query.report.c_str());
         if (src_val == -1)
         {
             sql_where.append_listf("1=0");
-            TRACE("rep_memo %s not found: adding AND 1=0\n", query.rep_memo.c_str());
+            TRACE("rep_memo %s not found: adding AND 1=0\n", query.report.c_str());
         } else {
             sql_where.append_listf("s.rep=%d", src_val);
-            TRACE("found rep_memo %s: adding AND s.rep=%d\n", query.rep_memo.c_str(), src_val);
+            TRACE("found rep_memo %s: adding AND s.rep=%d\n", query.report.c_str(), src_val);
         }
         has_where = true;
     }
@@ -435,19 +425,17 @@ bool DataQueryBuilder::add_attrfilter_where(const char* tbl)
 
 void DataQueryBuilder::build_order_by()
 {
-    if (modifiers & DBA_DB_MODIFIER_SORT_FOR_EXPORT)
-        sql_query.append(" ORDER BY d.id_station");
-    else
+    if (modifiers & DBA_DB_MODIFIER_BEST)
         sql_query.append(" ORDER BY s.lat, s.lon, s.ident");
+    else
+        sql_query.append(" ORDER BY d.id_station");
+
     if (!query_station_vars)
     {
         sql_query.append(", d.datetime");
         sql_query.append(", ltr.ltype1, ltr.l1, ltr.ltype2, ltr.l2, ltr.pind, ltr.p1, ltr.p2");
     }
-    // TODO: id_report should be at the end, so we get all the variables for a
-    // given report. If this has not been caught so far, are clients actually
-    // relying on ordering? If not, negotiate relaxing of ordering requirements (#19)
-    if (!(modifiers & DBA_DB_MODIFIER_SORT_FOR_EXPORT))
+    if (modifiers & DBA_DB_MODIFIER_BEST)
         sql_query.append(", s.rep");
     sql_query.append(", d.code");
 }
@@ -615,10 +603,10 @@ bool QueryBuilder::add_dt_where(const char* tbl)
     if (query_station_vars) return false;
 
     bool found = false;
-    if (!query.datetime.is_missing())
+    if (!query.dtrange.is_missing())
     {
-        Datetime dtmin = query.datetime.min;
-        Datetime dtmax = query.datetime.max;
+        Datetime dtmin = query.dtrange.min;
+        Datetime dtmax = query.dtrange.max;
         if (dtmin == dtmax)
         {
             // Add constraint on the exact date interval
@@ -724,10 +712,10 @@ bool QueryBuilder::add_repinfo_where(const char* tbl)
 {
     bool found = false;
 
-    if (query.prio_min != MISSING_INT || query.prio_max != MISSING_INT)
+    if (query.priomin != MISSING_INT || query.priomax != MISSING_INT)
     {
         // Filter the repinfo cache and build a IN query
-        std::vector<int> ids = db.repinfo().ids_by_prio(query);
+        std::vector<int> ids = tr->repinfo().ids_by_prio(query);
         if (ids.empty())
         {
             // No repinfo matches, so we just introduce a false value
@@ -746,16 +734,16 @@ bool QueryBuilder::add_repinfo_where(const char* tbl)
         found = true;
     }
 
-    if (!query.rep_memo.empty())
+    if (!query.report.empty())
     {
-        int src_val = db.repinfo().get_id(query.rep_memo.c_str());
+        int src_val = tr->repinfo().get_id(query.report.c_str());
         if (src_val == -1)
         {
             sql_where.append_listf("1=0");
-            TRACE("rep_memo %s not found: adding AND 1=0\n", query.rep_memo.c_str());
+            TRACE("rep_memo %s not found: adding AND 1=0\n", query.report.c_str());
         } else {
             sql_where.append_listf("%s.rep=%d", tbl, src_val);
-            TRACE("found rep_memo %s: adding AND %s.rep=%d\n", query.rep_memo.c_str(), tbl, (int)src_val);
+            TRACE("found rep_memo %s: adding AND %s.rep=%d\n", query.report.c_str(), tbl, (int)src_val);
         }
         found = true;
     }
