@@ -1,6 +1,5 @@
 #include "repinfo.h"
 #include "dballe/db/db.h"
-#include "dballe/record.h"
 #include "dballe/core/query.h"
 #include "dballe/core/csv.h"
 #include <wreport/error.h>
@@ -18,19 +17,6 @@ Repinfo::Repinfo(dballe::sql::Connection& conn)
 {
 }
 
-void Repinfo::to_record(int id, Record& rec)
-{
-    const repinfo::Cache* c = get_by_id(id);
-    if (c)
-    {
-        rec.setc("rep_memo", c->memo.c_str());
-        rec.seti("priority", c->prio);
-    } else {
-        rec.unset("rep_memo");
-        rec.unset("priority");
-    }
-}
-
 const char* Repinfo::get_rep_memo(int id)
 {
     if (const repinfo::Cache* c = get_by_id(id))
@@ -53,9 +39,9 @@ int Repinfo::get_id(const char* memo)
     return memo_idx[pos].id;
 }
 
-int Repinfo::get_priority(int id)
+int Repinfo::get_priority(const std::string& report)
 {
-    const repinfo::Cache* ri_entry = get_by_id(id);
+    const repinfo::Cache* ri_entry = get_by_memo(report.c_str());
     return ri_entry ? ri_entry->prio : INT_MAX;
 }
 
@@ -94,8 +80,8 @@ std::vector<int> Repinfo::ids_by_prio(const core::Query& q)
     for (std::vector<repinfo::Cache>::const_iterator i = cache.begin();
             i != cache.end(); ++i)
     {
-        if (q.prio_min != MISSING_INT && i->prio < q.prio_min) continue;
-        if (q.prio_max != MISSING_INT && i->prio > q.prio_max) continue;
+        if (q.priomin != MISSING_INT && i->prio < q.priomin) continue;
+        if (q.priomax != MISSING_INT && i->prio > q.priomax) continue;
         res.push_back(i->id);
     }
 
@@ -303,12 +289,14 @@ void Repinfo::update(const char* deffile, int* added, int* deleted, int* updated
     // Verify that we are not trying to delete a repinfo entry that is
     // in use
     for (const auto& entry : cache)
+    {
         /* Ensure that we are not deleting a repinfo entry that is already in use */
         if (!entry.memo.empty() && entry.new_memo.empty())
             if (id_use_count(entry.id, entry.memo.c_str()) > 0)
                 error_consistency::throwf(
                         "trying to delete repinfo entry %u,%s which is currently in use",
                         (unsigned)entry.id, entry.memo.c_str());
+    }
 
     // Perform the changes
 
