@@ -8,13 +8,6 @@
 #include <memory>
 
 namespace dballe {
-struct Record;
-struct Msg;
-
-namespace msg {
-struct Context;
-}
-
 namespace db {
 namespace v7 {
 namespace sqlite {
@@ -43,19 +36,17 @@ protected:
     void _dump(std::function<void(int, const Level&, const Trange&)> out) override;
 
 public:
-    SQLiteLevTr(dballe::sql::SQLiteConnection& conn);
+    SQLiteLevTr(v7::Transaction& tr, dballe::sql::SQLiteConnection& conn);
     SQLiteLevTr(const LevTr&) = delete;
     SQLiteLevTr(const LevTr&&) = delete;
     SQLiteLevTr& operator=(const SQLiteLevTr&) = delete;
     ~SQLiteLevTr();
 
-    void prefetch_ids(const std::set<int>& ids, std::function<void(int, const LevTrDesc&)> dest) override;
-    void prefetch_same_level(int id, std::function<void(int, const LevTrDesc&)> dest) override;
-    levtrs_t::iterator lookup_id(State& st, int id) override;
-    levtrs_t::iterator obtain_id(State& state, const LevTrDesc& desc) override;
+    void prefetch_ids(Tracer<>& trc, const std::set<int>& id) override;
+    const LevTrEntry* lookup_id(Tracer<>& trc, int id) override;
+    int obtain_id(Tracer<>& trc, const LevTrEntry& desc) override;
 };
 
-
 }
 }
 }
