@@ -1,4 +1,6 @@
 #include "station.h"
+#include "dballe/core/values.h"
+#include "transaction.h"
 
 using namespace wreport;
 using namespace dballe::db;
@@ -8,23 +10,13 @@ namespace dballe {
 namespace db {
 namespace v7 {
 
-Station::~Station()
+Station::Station(v7::Transaction& tr)
+    : tr(tr)
 {
 }
 
-stations_t::iterator Station::get_id(State& st, const StationDesc& desc)
+Station::~Station()
 {
-    auto res = st.stations.find(desc);
-    if (res != st.stations.end())
-        return res;
-
-    StationState state;
-    if (maybe_get_id(desc, &state.id))
-    {
-        state.is_new = false;
-        return st.add_station(desc, state);
-    }
-    throw error_notfound("station not found in the database");
 }
 
 void Station::dump(FILE* out)
