@@ -1,28 +1,22 @@
 #ifndef DBALLE_FILE_H
 #define DBALLE_FILE_H
 
-#include <dballe/types.h>
+#include <dballe/fwd.h>
 #include <memory>
 #include <string>
+#include <functional>
+#include <iosfwd>
 
 namespace dballe {
 
-struct BinaryMessage;
-
 /**
  * File object for doing I/O on binary message streams.
  *
  * It provides a unified interface to read and write messages to files.
  */
-struct File
+class File
 {
-    /// Supported encodings
-    typedef enum {
-        BUFR = 0,
-        CREX = 1,
-        AOF = 2,
-    } Encoding;
-
+public:
     virtual ~File();
 
     /// Get the file pathname
@@ -31,6 +25,11 @@ struct File
     /// Get the file encoding
     virtual Encoding encoding() const = 0;
 
+    /**
+     * Close the underlying file
+     */
+    virtual void close() = 0;
+
     /**
      * Read a message from the file.
      *
@@ -128,10 +127,11 @@ struct File
 };
 
 /// Binary message
-struct BinaryMessage
+class BinaryMessage
 {
+public:
     /// Format of the binary data
-    File::Encoding encoding;
+    Encoding encoding;
 
     /// Binary message data
     std::string data;
@@ -149,13 +149,21 @@ struct BinaryMessage
     /// Index of the message from the beginning of the file
     int index = MISSING_INT;
 
-    BinaryMessage(File::Encoding encoding)
+    BinaryMessage(Encoding encoding)
         : encoding(encoding) {}
+    BinaryMessage(const BinaryMessage&) = default;
+    BinaryMessage(BinaryMessage&&) = default;
+    BinaryMessage& operator=(const BinaryMessage&) = default;
+    BinaryMessage& operator=(BinaryMessage&&) = default;
 
     /// Return true if the message is not empty
     operator bool() const;
 };
 
+
+/// Serialize Encoding
+std::ostream& operator<<(std::ostream&, const dballe::Encoding&);
+
 }
 
 #endif
