@@ -1,5 +1,4 @@
 #include "tests.h"
-#include "codec.h"
 #include "wr_codec.h"
 #include <dballe/core/csv.h>
 #include "context.h"
@@ -9,6 +8,7 @@
 #include <wreport/tableinfo.h>
 #include <wreport/conv.h>
 #include <wreport/notes.h>
+#include <wreport/codetables.h>
 #include <cstring>
 #include <cmath>
 #include <unistd.h>
@@ -87,43 +87,26 @@ const char* crex_files[] = {
     NULL
 };
 
-const char* aof_files[] = {
-	"aof/obs1-11.0.aof",
-	"aof/obs1-14.63.aof",
-	"aof/obs1-21.1.aof",
-	"aof/obs1-24.2104.aof",
-	"aof/obs1-24.34.aof",
-	"aof/obs2-144.2198.aof",
-	"aof/obs2-244.0.aof",
-	"aof/obs2-244.1.aof",
-	"aof/obs4-165.2027.aof",
-	"aof/obs5-35.61.aof",
-	"aof/obs5-36.30.aof",
-	"aof/obs6-32.1573.aof",
-	"aof/obs6-32.0.aof",
-	"aof/aof_27-2-144.aof",
-	"aof/aof_28-2-144.aof",
-	"aof/aof_27-2-244.aof",
-	"aof/aof_28-2-244.aof",
-	"aof/missing-cloud-h.aof",
-	"aof/brokenamdar.aof",
-	"aof/aof-undersealevel.aof",
-	NULL,
-};
-
-Messages read_msgs(const char* filename, File::Encoding type, const msg::Importer::Options& opts)
+impl::Messages read_msgs(const char* filename, Encoding type, const ImporterOptions& opts)
 {
     BinaryMessage raw = wcallchecked(read_rawmsg(filename, type));
-    std::unique_ptr<msg::Importer> importer = msg::Importer::create(type, opts);
+    std::unique_ptr<Importer> importer = Importer::create(type, opts);
     return importer->from_binary(raw);
 }
 
-Messages read_msgs_csv(const char* filename)
+impl::Messages read_msgs(const char* filename, Encoding type, const std::string& opts)
+{
+    BinaryMessage raw = wcallchecked(read_rawmsg(filename, type));
+    std::unique_ptr<Importer> importer = Importer::create(type, opts);
+    return importer->from_binary(raw);
+}
+
+impl::Messages read_msgs_csv(const char* filename)
 {
     std::string fname = datafile(filename);
     CSVReader reader(fname);
 
-    Messages msgs = msg::messages_from_csv(reader);
+    impl::Messages msgs = impl::msg::messages_from_csv(reader);
     if (msgs.empty())
     {
         std::stringstream ss;
@@ -133,10 +116,10 @@ Messages read_msgs_csv(const char* filename)
     return msgs;
 }
 
-unique_ptr<Bulletin> export_msgs(File::Encoding enctype, const Messages& in, const std::string& tag, const dballe::msg::Exporter::Options& opts)
+unique_ptr<Bulletin> export_msgs(Encoding enctype, const impl::Messages& in, const std::string& tag, const ExporterOptions& opts)
 {
     try {
-        std::unique_ptr<msg::Exporter> exporter(msg::Exporter::create(enctype, opts));
+        std::unique_ptr<Exporter> exporter(Exporter::create(enctype, opts));
         return exporter->to_bulletin(in);
     } catch (std::exception& e) {
         dballe::tests::dump("bul-" + tag, in);
@@ -158,24 +141,24 @@ void track_different_msgs(const Message& msg1, const Message& msg2, const std::s
 	cerr << "Wrote mismatching messages to " << fname1 << " and " << fname2 << endl;
 }
 
-void track_different_msgs(const Messages& msgs1, const Messages& msgs2, const std::string& prefix)
+void track_different_msgs(const impl::Messages& msgs1, const impl::Messages& msgs2, const std::string& prefix)
 {
     dump(prefix + "1", msgs1, "first message");
     dump(prefix + "2", msgs2, "second message");
 }
 
-void ActualMessage::is_undef(int shortcut) const
+void ActualMessage::is_undef(const impl::Shortcut& shortcut) const
 {
-    const Var* var = Msg::downcast(_actual).find_by_id(shortcut);
+    const Var* var = impl::Message::downcast(_actual).get(shortcut);
     if (!var || !var->isset()) return;
     std::stringstream ss;
     ss << "value is " << var->enqc() << " instead of being undefined";
     throw TestFailed(ss.str());
 }
 
-const Var& want_var(const Message& msg, int shortcut)
+const Var& want_var(const Message& msg, const impl::Shortcut& shortcut)
 {
-    const Var* var = Msg::downcast(msg).find_by_id(shortcut);
+    const Var* var = impl::Message::downcast(msg).get(shortcut);
     if (!var)
         throw TestFailed("value is missing");
     if (!var->isset())
@@ -185,7 +168,7 @@ const Var& want_var(const Message& msg, int shortcut)
 
 const Var& want_var(const Message& msg, wreport::Varcode code, const dballe::Level& lev, const dballe::Trange& tr)
 {
-    const Var* var = msg.get(code, lev, tr);
+    const Var* var = msg.get(lev, tr, code);
     if (!var)
         throw TestFailed("value is missing");
     if (!var->isset())
@@ -193,7 +176,7 @@ const Var& want_var(const Message& msg, wreport::Varcode code, const dballe::Lev
     return *var;
 }
 
-void dump(const std::string& tag, const Msg& msg, const std::string& desc)
+void dump(const std::string& tag, const impl::Message& msg, const std::string& desc)
 {
     string fname = "/tmp/" + tag + ".txt";
     FILE* out = fopen(fname.c_str(), "w");
@@ -206,12 +189,12 @@ void dump(const std::string& tag, const Msg& msg, const std::string& desc)
     cerr << desc << " saved in " << fname << endl;
 }
 
-void dump(const std::string& tag, const Messages& msgs, const std::string& desc)
+void dump(const std::string& tag, const impl::Messages& msgs, const std::string& desc)
 {
     string fname = "/tmp/" + tag + ".txt";
     FILE* out = fopen(fname.c_str(), "w");
     try {
-        msgs.print(out);
+        impl::msg::messages_print(msgs, out);
     } catch (std::exception& e) {
         fprintf(out, "Dump interrupted: %s\n", e.what());
     }
@@ -270,7 +253,7 @@ void MessageTweakers::add(MessageTweaker* tweak)
     tweaks.push_back(tweak);
 }
 
-void MessageTweakers::apply(Messages& msgs)
+void MessageTweakers::apply(impl::Messages& msgs)
 {
     for (vector<MessageTweaker*>::iterator i = tweaks.begin(); i != tweaks.end(); ++i)
         (*i)->tweak(msgs);
@@ -278,22 +261,20 @@ void MessageTweakers::apply(Messages& msgs)
 
 namespace tweaks {
 
-void StripAttrs::tweak(Messages& msgs)
+void StripAttrs::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
     {
-        Msg& m = Msg::downcast(mi);
-        for (vector<msg::Context*>::iterator ci = m.data.begin(); ci != m.data.end(); ++ci)
-        {
-            msg::Context& c = **ci;
-            for (vector<wreport::Var*>::iterator vi = c.data.begin(); vi != c.data.end(); ++vi)
-            {
-                Var& v = **vi;
-                for (vector<wreport::Varcode>::const_iterator i = codes.begin();
-                        i != codes.end(); ++i)
-                    v.unseta(*i);
-            }
-        }
+        auto m = impl::Message::downcast(mi);
+
+        for (auto& var: m->station_data)
+            for (auto code: codes)
+                var->unseta(code);
+
+        for (auto& ctx: m->data)
+            for (auto& var: ctx.values)
+                for (auto code: codes)
+                    var->unseta(code);
     }
 }
 
@@ -309,76 +290,64 @@ StripContextAttrs::StripContextAttrs()
     codes.push_back(WR_VAR(0, 4, 194));
 }
 
-void StripSubstituteAttrs::tweak(Messages& msgs)
+void StripSubstituteAttrs::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
     {
-        Msg& m = Msg::downcast(mi);
-        for (vector<msg::Context*>::iterator ci = m.data.begin(); ci != m.data.end(); ++ci)
-        {
-            msg::Context& c = **ci;
-            for (vector<wreport::Var*>::iterator vi = c.data.begin(); vi != c.data.end(); ++vi)
-            {
-                Var& v = **vi;
-                v.unseta(v.code());
-            }
-        }
+        auto m = impl::Message::downcast(mi);
+        for (auto& var: m->station_data)
+            var->unseta(var->code());
+
+        for (auto& ctx: m->data)
+            for (auto& var: ctx.values)
+                var->unseta(var->code());
     }
 }
 
-void StripVars::tweak(Messages& msgs)
+void StripVars::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
     {
-        Msg& m = Msg::downcast(mi);
-        for (vector<msg::Context*>::iterator ci = m.data.begin(); ci != m.data.end(); )
+        auto m = impl::Message::downcast(mi);
+
+        for (const auto& code: codes)
+            m->station_data.unset(code);
+
+        for (auto ci = m->data.begin(); ci != m->data.end(); )
         {
-            msg::Context& c = **ci;
-            for (vector<wreport::Varcode>::const_iterator i = codes.begin();
-                    i != codes.end(); ++i)
-                c.remove(*i);
-            if (c.data.empty())
-                ci = m.data.erase(ci);
+            for (const auto& code: codes)
+                ci->values.unset(code);
+            if (ci->values.empty())
+                ci = m->data.erase(ci);
             else
                 ++ci;
         }
     }
 }
 
-StripDatetimeVars::StripDatetimeVars()
-{
-    codes.push_back(WR_VAR(0, 4, 1));
-    codes.push_back(WR_VAR(0, 4, 2));
-    codes.push_back(WR_VAR(0, 4, 3));
-    codes.push_back(WR_VAR(0, 4, 4));
-    codes.push_back(WR_VAR(0, 4, 5));
-    codes.push_back(WR_VAR(0, 4, 6));
-}
-
 RoundLegacyVars::RoundLegacyVars() : table(NULL)
 {
     table = Vartable::get_bufr(BufrTableID(0, 0, 0, 14, 0));
 }
 
-void RoundLegacyVars::tweak(Messages& msgs)
+void RoundLegacyVars::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
     {
-        Msg& m = Msg::downcast(mi);
-        for (vector<msg::Context*>::iterator ci = m.data.begin(); ci != m.data.end(); ++ci)
+        auto m = impl::Message::downcast(mi);
+        for (auto& ctx: m->data)
         {
-            msg::Context& c = **ci;
-            if (Var* var = c.edit(WR_VAR(0, 12, 101)))
+            if (Var* var = ctx.values.maybe_var(WR_VAR(0, 12, 101)))
             {
                 Var var1(table->query(WR_VAR(0, 12, 1)), *var);
                 var->set(var1);
             }
-            if (Var* var = c.edit(WR_VAR(0, 12, 103)))
+            if (Var* var = ctx.values.maybe_var(WR_VAR(0, 12, 103)))
             {
                 Var var1(table->query(WR_VAR(0, 12, 3)), *var);
                 var->set(var1);
             }
-            if (Var* var = c.edit(WR_VAR(0,  7,  30)))
+            if (Var* var = ctx.values.maybe_var(WR_VAR(0,  7,  30)))
             {
                 Var var1(table->query(WR_VAR(0, 7, 1)), *var);
                 var->set(var1);
@@ -387,89 +356,89 @@ void RoundLegacyVars::tweak(Messages& msgs)
     }
 }
 
-void RemoveSynopWMOOnlyVars::tweak(Messages& msgs)
+void RemoveSynopWMOOnlyVars::tweak(impl::Messages& msgs)
 {
     int seen_tprec_trange = MISSING_INT;
     for (auto& mi: msgs)
     {
-        Msg& m = Msg::downcast(mi);
+        auto m = impl::Message::downcast(mi);
         // Remove all 'cloud drift' levels
-        for (int i = 1; m.remove_context(Level::cloud(260, i), Trange::instant()); ++i)
+        for (int i = 1; m->remove_context(Level::cloud(260, i), Trange::instant()); ++i)
             ;
         // Remove all 'cloud elevation' levels
-        for (int i = 1; m.remove_context(Level::cloud(261, i), Trange::instant()); ++i)
+        for (int i = 1; m->remove_context(Level::cloud(261, i), Trange::instant()); ++i)
             ;
         // Remove all 'cloud direction and elevation' levels
-        for (int i = 1; m.remove_context(Level::cloud(262, i), Trange::instant()); ++i)
+        for (int i = 1; m->remove_context(Level::cloud(262, i), Trange::instant()); ++i)
             ;
         // Remove all 'cloud below' levels
-        for (int i = 1; m.remove_context(Level::cloud(263, i), Trange::instant()); ++i)
+        for (int i = 1; m->remove_context(Level::cloud(263, i), Trange::instant()); ++i)
             ;
-        for (vector<msg::Context*>::iterator ci = m.data.begin(); ci != m.data.end(); )
+        for (auto ci = m->data.begin(); ci != m->data.end(); )
         {
-            msg::Context& c = **ci;
-            c.remove(WR_VAR(0, 20, 62)); // State of the ground (with/without snow)
-            c.remove(WR_VAR(0, 14, 31)); // Total sunshine
-            c.remove(WR_VAR(0, 13, 33)); // Evaporation/evapotranspiration
-            c.remove(WR_VAR(0, 12,121)); // Ground minimum temperature
-            c.remove(WR_VAR(0, 11, 43)); // Maximum wind gust
-            c.remove(WR_VAR(0, 11, 41)); // Maximum wind gust
-            c.remove(WR_VAR(0, 10,  8)); // Geopotential
-            c.remove(WR_VAR(0,  7, 31)); // Height of barometer
-            c.remove(WR_VAR(0,  2,  4)); // Type of instrumentation for evaporation measurement
-            c.remove(WR_VAR(0,  2,  2)); // Type of instrumentation for wind measurement
-            c.remove(WR_VAR(0,  1, 19)); // Long station or site name
-            c.remove(WR_VAR(0, 14,  2)); // Radiation data
-            c.remove(WR_VAR(0, 14,  4)); // Radiation data
-            c.remove(WR_VAR(0, 14, 16)); // Radiation data
-            c.remove(WR_VAR(0, 14, 28)); // Radiation data
-            c.remove(WR_VAR(0, 14, 29)); // Radiation data
-            c.remove(WR_VAR(0, 14, 30)); // Radiation data
-            if (c.find(WR_VAR(0, 13, 11)))
+            impl::msg::Context& c = *ci;
+            c.values.unset(WR_VAR(0, 20, 62)); // State of the ground (with/without snow)
+            c.values.unset(WR_VAR(0, 14, 31)); // Total sunshine
+            c.values.unset(WR_VAR(0, 13, 33)); // Evaporation/evapotranspiration
+            c.values.unset(WR_VAR(0, 12,121)); // Ground minimum temperature
+            c.values.unset(WR_VAR(0, 11, 43)); // Maximum wind gust
+            c.values.unset(WR_VAR(0, 11, 41)); // Maximum wind gust
+            c.values.unset(WR_VAR(0, 10,  8)); // Geopotential
+            c.values.unset(WR_VAR(0,  7, 31)); // Height of barometer
+            c.values.unset(WR_VAR(0,  2,  4)); // Type of instrumentation for evaporation measurement
+            c.values.unset(WR_VAR(0,  2,  2)); // Type of instrumentation for wind measurement
+            c.values.unset(WR_VAR(0,  1, 19)); // Long station or site name
+            c.values.unset(WR_VAR(0, 14,  2)); // Radiation data
+            c.values.unset(WR_VAR(0, 14,  4)); // Radiation data
+            c.values.unset(WR_VAR(0, 14, 16)); // Radiation data
+            c.values.unset(WR_VAR(0, 14, 28)); // Radiation data
+            c.values.unset(WR_VAR(0, 14, 29)); // Radiation data
+            c.values.unset(WR_VAR(0, 14, 30)); // Radiation data
+            if (c.values.maybe_var(WR_VAR(0, 13, 11)))
             {
                 // Keep only one total precipitation measurement common
                 // to all subsets
                 if (seen_tprec_trange == MISSING_INT)
                     seen_tprec_trange = c.trange.p2;
                 else if (c.trange.p2 != seen_tprec_trange)
-                    c.remove(WR_VAR(0, 13, 11));
+                    c.values.unset(WR_VAR(0, 13, 11));
             }
             if (c.trange == Trange(4, 0, 86400))
-                c.remove(WR_VAR(0, 10, 60)); // 24h pressure change
+                c.values.unset(WR_VAR(0, 10, 60)); // 24h pressure change
             if (c.trange.pind == 2 || c.trange.pind == 3)
-                c.remove(WR_VAR(0, 12, 101)); // min and max temperature
-            if (c.data.empty())
-                ci = m.data.erase(ci);
+                c.values.unset(WR_VAR(0, 12, 101)); // min and max temperature
+            if (c.values.empty())
+                ci = m->data.erase(ci);
             else
                 ++ci;
         }
     }
 }
 
-void RemoveTempWMOOnlyVars::tweak(Messages& msgs)
+void RemoveTempWMOOnlyVars::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
     {
-        Msg& m = Msg::downcast(mi);
-        for (vector<msg::Context*>::iterator ci = m.data.begin(); ci != m.data.end(); )
+        auto m = impl::Message::downcast(mi);
+        for (auto ci = m->data.begin(); ci != m->data.end(); )
         {
-            msg::Context& c = **ci;
-            c.remove(WR_VAR(0, 22, 43)); // Sea/water temperature
-            c.remove(WR_VAR(0, 11, 62)); // Absolute wind shear layer above
-            c.remove(WR_VAR(0, 11, 61)); // Absolute wind shear layer below
-            c.remove(WR_VAR(0,  7, 31)); // Height of barometer above MSL
-            c.remove(WR_VAR(0,  7,  7)); // Height (of release above MSL)
-            c.remove(WR_VAR(0,  6, 15)); // Longitude displacement
-            c.remove(WR_VAR(0,  5, 15)); // Latitude displacement
-            c.remove(WR_VAR(0,  4, 86)); // Long time period or displacement
-            c.remove(WR_VAR(0,  4,  6)); // Second
-            c.remove(WR_VAR(0,  2, 14)); // Tracking technique / status of system
-            c.remove(WR_VAR(0,  2, 13)); // Solar and infrared radiation correction
-            c.remove(WR_VAR(0,  2, 12)); // Radiosonde computational method
-            c.remove(WR_VAR(0,  2,  3)); // Type of measuring equipment
+            impl::msg::Context& c = *ci;
+            c.values.unset(WR_VAR(0, 22, 43)); // Sea/water temperature
+            c.values.unset(WR_VAR(0, 11, 62)); // Absolute wind shear layer above
+            c.values.unset(WR_VAR(0, 11, 61)); // Absolute wind shear layer below
+            c.values.unset(WR_VAR(0,  7, 31)); // Height of barometer above MSL
+            c.values.unset(WR_VAR(0,  7,  7)); // Height (of release above MSL)
+            c.values.unset(WR_VAR(0,  6, 15)); // Longitude displacement
+            c.values.unset(WR_VAR(0,  5, 15)); // Latitude displacement
+            c.values.unset(WR_VAR(0,  4, 86)); // Long time period or displacement
+            c.values.unset(WR_VAR(0,  4,  6)); // Second
+            c.values.unset(WR_VAR(0,  2, 14)); // Tracking technique / status of system
+            c.values.unset(WR_VAR(0,  2, 13)); // Solar and infrared radiation correction
+            c.values.unset(WR_VAR(0,  2, 12)); // Radiosonde computational method
+            c.values.unset(WR_VAR(0,  2,  3)); // Type of measuring equipment
 
-            if (c.data.empty())
-                ci = m.data.erase(ci);
+            if (c.values.empty())
+                ci = m->data.erase(ci);
             else
                 ++ci;
         }
@@ -481,31 +450,27 @@ RemoveOddTempTemplateOnlyVars::RemoveOddTempTemplateOnlyVars()
     codes.push_back(WR_VAR(0, 2, 12)); // Radiosonde computational method
 }
 
-void RemoveSynopWMOOddprec::tweak(Messages& msgs)
+void RemoveSynopWMOOddprec::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
-    {
-        Msg& m = Msg::downcast(mi);
-        m.remove_context(Level(1), Trange(1, 0));
-    }
+        impl::Message::downcast(mi)->remove_context(Level(1), Trange(1, 0));
 }
 
-void TruncStName::tweak(Messages& msgs)
+void TruncStName::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
     {
-        Msg& m = Msg::downcast(mi);
-        if (msg::Context* c = m.edit_context(Level(), Trange()))
-            if (const Var* orig = c->find(WR_VAR(0, 1, 19)))
-                if (orig->isset())
-                {
-                    const char* val = orig->enqc();
-                    char buf[21];
-                    strncpy(buf, val, 20);
-                    buf[19] = '>';
-                    buf[20] = 0;
-                    c->set(Var(orig->info(), buf));
-                }
+        auto m = impl::Message::downcast(mi);
+        if (Var* orig = m->station_data.maybe_var(WR_VAR(0, 1, 19)))
+            if (orig->isset())
+            {
+                const char* val = orig->enqc();
+                char buf[21];
+                strncpy(buf, val, 20);
+                buf[19] = '>';
+                buf[20] = 0;
+                orig->set(buf);
+            }
     }
 }
 
@@ -513,23 +478,19 @@ RoundGeopotential::RoundGeopotential()
 {
     table = Vartable::get_bufr(BufrTableID(0, 0, 0, 14, 0));
 }
-void RoundGeopotential::tweak(Messages& msgs)
+void RoundGeopotential::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
     {
-        Msg& m = Msg::downcast(mi);
-        for (vector<msg::Context*>::iterator ci = m.data.begin(); ci != m.data.end(); ++ci)
+        auto m = impl::Message::downcast(mi);
+        for (auto& ctx: m->data)
         {
-            msg::Context& c = **ci;
-            if (Var* orig = c.edit(WR_VAR(0, 10, 8)))
+            if (Var* orig = ctx.values.maybe_var(WR_VAR(0, 10, 8)))
             {
                 // Convert to B10009 (new GTS TEMP templates)
                 Var var2(table->query(WR_VAR(0, 10, 9)), *orig);
-                // Convert to B10008 (used for geopotential by DB-All.e)
-                Var var3(table->query(WR_VAR(0, 10, 8)), var2);
-                // Convert back to B10003
-                Var var4(table->query(WR_VAR(0, 10, 3)), var3);
-                orig->set(var4);
+                // Convert back to B10008 (used for geopotential by DB-All.e)
+                orig->set(var2);
             }
         }
     }
@@ -539,32 +500,34 @@ HeightToGeopotential::HeightToGeopotential()
 {
     table = Vartable::get_bufr(BufrTableID(0, 0, 0, 14, 0));
 }
-void HeightToGeopotential::tweak(Messages& msgs)
+void HeightToGeopotential::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
     {
-        Msg& m = Msg::downcast(mi);
-        for (vector<msg::Context*>::iterator ci = m.data.begin(); ci != m.data.end(); ++ci)
+        auto m = impl::Message::downcast(mi);
+        for (auto& ctx: m->data)
         {
-            msg::Context& c = **ci;
-            if (c.level.ltype1 != 102) continue;
-            Var var(table->query(WR_VAR(0, 10, 8)), round(c.level.l1 * 9.807 / 10) * 10);
-            c.set(var);
+            if (ctx.level.ltype1 != 102) continue;
+            Var var(table->query(WR_VAR(0, 10, 8)), round(ctx.level.l1 * 9.807 / 10) * 10);
+            ctx.values.set(var);
         }
     }
 }
 
-void RoundVSS::tweak(Messages& msgs)
+void RoundVSS::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
     {
-        Msg& m = Msg::downcast(mi);
-        for (vector<msg::Context*>::iterator ci = m.data.begin(); ci != m.data.end(); ++ci)
-        {
-            msg::Context& c = **ci;
-            if (Var* orig = c.edit(WR_VAR(0, 8, 42)))
-                orig->seti(convert_BUFR08001_to_BUFR08042(convert_BUFR08042_to_BUFR08001(orig->enqi())));
-        }
+        auto m = impl::Message::downcast(mi);
+        for (auto& ctx: m->data)
+            if (Var* orig = ctx.values.maybe_var(WR_VAR(0, 8, 42)))
+            {
+                int val = convert_BUFR08001_to_BUFR08042(convert_BUFR08042_to_BUFR08001(orig->enqi()));
+                if (val == wreport::BUFR08042::ALL_MISSING)
+                    orig->unset();
+                else
+                    orig->seti(val);
+            }
     }
 }
 
@@ -572,18 +535,15 @@ RemoveContext::RemoveContext(const Level& lev, const Trange& tr)
     : lev(lev), tr(tr)
 {
 }
-void RemoveContext::tweak(Messages& msgs)
+void RemoveContext::tweak(impl::Messages& msgs)
 {
     for (auto& mi: msgs)
-    {
-        Msg& m = Msg::downcast(mi);
-        m.remove_context(lev, tr);
-    }
+        impl::Message::downcast(mi)->remove_context(lev, tr);
 }
 
 }
 
-TestMessage::TestMessage(File::Encoding type, const std::string& name)
+TestMessage::TestMessage(Encoding type, const std::string& name)
     : name(name), type(type), raw(type)
 {
 }
@@ -593,28 +553,28 @@ TestMessage::~TestMessage()
     delete bulletin;
 }
 
-void TestMessage::read_from_file(const std::string& fname, const msg::Importer::Options& input_opts)
+void TestMessage::read_from_file(const std::string& fname, const ImporterOptions& input_opts)
 {
     read_from_raw(read_rawmsg(fname.c_str(), type), input_opts);
 }
 
-void TestMessage::read_from_raw(const BinaryMessage& msg, const msg::Importer::Options& input_opts)
+void TestMessage::read_from_raw(const BinaryMessage& msg, const ImporterOptions& input_opts)
 {
-    std::unique_ptr<msg::Importer> importer(msg::Importer::create(type, input_opts));
+    std::unique_ptr<Importer> importer(Importer::create(type, input_opts));
     raw = msg;
     switch (type)
     {
-        case File::BUFR: bulletin = BufrBulletin::decode(raw.data).release(); break;
-        case File::CREX: bulletin = CrexBulletin::decode(raw.data).release(); break;
+        case Encoding::BUFR: bulletin = BufrBulletin::decode(raw.data).release(); break;
+        case Encoding::CREX: bulletin = CrexBulletin::decode(raw.data).release(); break;
         default: throw wreport::error_unimplemented("Unsupported message type");
     }
     msgs = importer->from_binary(raw);
 }
 
-void TestMessage::read_from_msgs(const Messages& _msgs, const msg::Exporter::Options& export_opts)
+void TestMessage::read_from_msgs(const impl::Messages& _msgs, const ExporterOptions& export_opts)
 {
     // Export
-    std::unique_ptr<msg::Exporter> exporter(msg::Exporter::create(type, export_opts));
+    std::unique_ptr<Exporter> exporter(Exporter::create(type, export_opts));
     msgs = _msgs;
     delete bulletin;
     bulletin = exporter->to_bulletin(msgs).release();
@@ -642,12 +602,12 @@ void TestMessage::dump() const
 
     fname = basename + ".messages";
     out = fopen(fname.c_str(), "w");
-    msgs.print(out);
+    impl::msg::messages_print(msgs, out);
     fclose(out);
     cerr << name << " interpreted saved in " << fname << endl;
 }
 
-TestCodec::TestCodec(const std::string& fname, File::Encoding type)
+TestCodec::TestCodec(const std::string& fname, Encoding type)
     : fname(fname), type(type)
 {
 }
@@ -655,7 +615,6 @@ TestCodec::TestCodec(const std::string& fname, File::Encoding type)
 void TestCodec::configure_ecmwf_to_wmo_tweaks()
 {
     after_convert_import.add(new tweaks::StripQCAttrs);
-    after_convert_import.add(new tweaks::StripDatetimeVars);
 }
 
 void TestCodec::do_compare(const TestMessage& msg1, const TestMessage& msg2)
@@ -664,7 +623,7 @@ void TestCodec::do_compare(const TestMessage& msg1, const TestMessage& msg2)
     {
         stringstream str;
         notes::Collect c(str);
-        int diffs = msg1.msgs.diff(msg2.msgs);
+        int diffs = impl::msg::messages_diff(msg1.msgs, msg2.msgs);
         if (diffs)
         {
             msg1.dump();
@@ -719,8 +678,8 @@ void TestCodec::run_reimport()
 
     if (!expected_template.empty())
     {
-        std::unique_ptr<msg::Exporter> exporter(msg::Exporter::create(type, output_opts));
-        msg::WRExporter* exp = dynamic_cast<msg::WRExporter*>(exporter.get());
+        std::unique_ptr<Exporter> exporter(Exporter::create(type, output_opts));
+        impl::msg::WRExporter* exp = dynamic_cast<impl::msg::WRExporter*>(exporter.get());
         auto tpl = exp->infer_template(orig.msgs);
         wassert(actual(tpl->name()) == expected_template);
     }
@@ -768,7 +727,7 @@ void TestCodec::run_convert(const std::string& tplname)
 
     // Export
     if (verbose) cerr << "Exporting with template " << tplname << " and options " << output_opts.to_string() << endl;
-    msg::Exporter::Options output_opts = this->output_opts;
+    impl::ExporterOptions output_opts = this->output_opts;
     output_opts.template_name = tplname;
     TestMessage exported(type, "exported");
     try {
