@@ -1,26 +1,8 @@
-/*
- * Copyright (C) 2005--2015  ARPA-SIM <urpsim@smr.arpa.emr.it>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
- *
- * Author: Enrico Zini <enrico@enricozini.com>
- */
-
 #include "common.h"
-#include "msg/wr_codec.h"
+#include "dballe/core/shortcuts.h"
+#include "dballe/msg/wr_codec.h"
+#include "dballe/msg/context.h"
 #include <wreport/bulletin.h>
-#include "msg/context.h"
 
 using namespace wreport;
 using namespace std;
@@ -38,6 +20,7 @@ using namespace std;
 #define SYNOP_ECMWF_AUTO_DESC "Synop ECMWF land auto (0.3)"
 
 namespace dballe {
+namespace impl {
 namespace msg {
 namespace wr {
 
@@ -53,7 +36,7 @@ struct Synop : public Template
     const msg::Context* c_radiation24;
     const msg::Context* c_tchange;
 
-    Synop(const Exporter::Options& opts, const Messages& msgs)
+    Synop(const dballe::ExporterOptions& opts, const Messages& msgs)
         : Template(opts, msgs) {}
 
     void scan_levels()
@@ -66,29 +49,27 @@ struct Synop : public Template
         c_tchange = NULL;
 
         // Scan message finding context for the data that follow
-        for (std::vector<msg::Context*>::const_iterator i = msg->data.begin();
-                i != msg->data.end(); ++i)
+        for (const auto& ctx: msg->data)
         {
-            const msg::Context* c = *i;
-            synop.scan_context(*c);
-            switch (c->level.ltype1)
+            synop.scan_context(ctx);
+            switch (ctx.level.ltype1)
             {
                 case 1:
-                    switch (c->trange.pind)
+                    switch (ctx.trange.pind)
                     {
                         case 1:
-                            for (const auto v: c->data)
+                            for (const auto v: ctx.values)
                             {
                                 switch (v->code())
                                 {
                                     case WR_VAR(0, 14, 31):
                                         if (!c_sunshine1)
-                                            c_sunshine1 = c;
+                                            c_sunshine1 = &ctx;
                                         else if (!c_sunshine2)
-                                            c_sunshine2 = c;
+                                            c_sunshine2 = &ctx;
                                         break;
                                     case WR_VAR(0, 13, 33):
-                                        c_evapo = c;
+                                        c_evapo = &ctx;
                                         break;
                                     case WR_VAR(0, 14,  2):
                                     case WR_VAR(0, 14,  4):
@@ -96,13 +77,13 @@ struct Synop : public Template
                                     case WR_VAR(0, 14, 28):
                                     case WR_VAR(0, 14, 29):
                                     case WR_VAR(0, 14, 30):
-                                        switch (c->trange.p2)
+                                        switch (ctx.trange.p2)
                                         {
                                             case  1 * 3600:
-                                                c_radiation1 = c;
+                                                c_radiation1 = &ctx;
                                                 break;
                                             case 24 * 3600:
-                                                c_radiation24 = c;
+                                                c_radiation24 = &ctx;
                                                 break;
                                         }
                                         break;
@@ -110,8 +91,8 @@ struct Synop : public Template
                             }
                             break;
                         case 4:
-                            if (c->find(WR_VAR(0, 12, 49)))
-                                c_tchange = c;
+                            if (ctx.values.maybe_var(WR_VAR(0, 12, 49)))
+                                c_tchange = &ctx;
                             break;
                     }
                     break;
@@ -119,7 +100,7 @@ struct Synop : public Template
         }
     }
 
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         Template::to_subset(msg, subset);
         synop.init(msg, subset);
@@ -133,7 +114,7 @@ struct SynopECMWF : public Synop
     bool is_crex;
     Varcode prec_code;
 
-    SynopECMWF(const Exporter::Options& opts, const Messages& msgs)
+    SynopECMWF(const dballe::ExporterOptions& opts, const Messages& msgs)
         : Synop(opts, msgs) {}
 
     void add_prec()
@@ -153,7 +134,7 @@ struct SynopECMWF : public Synop
             subset->store_variable_undef(prec_code);
     }
 
-    virtual void setupBulletin(wreport::Bulletin& bulletin)
+    void setupBulletin(wreport::Bulletin& bulletin) override
     {
         Synop::setupBulletin(bulletin);
 
@@ -169,16 +150,16 @@ struct SynopECMWF : public Synop
         prec_code = 0;
         for (Messages::const_iterator mi = msgs.begin(); prec_code == 0 && mi != msgs.end(); ++mi)
         {
-            const Msg& msg = Msg::downcast(*mi);
-            if (msg.get_tot_prec24_var() != NULL)
+            auto msg = Message::downcast(*mi);
+            if (msg->get_tot_prec24_var() != NULL)
                 prec_code = WR_VAR(0, 13, 23);
-            else if (msg.get_tot_prec12_var() != NULL)
+            else if (msg->get_tot_prec12_var() != NULL)
                 prec_code = WR_VAR(0, 13, 22);
-            else if (msg.get_tot_prec6_var() != NULL)
+            else if (msg->get_tot_prec6_var() != NULL)
                 prec_code = WR_VAR(0, 13, 21);
-            else if (msg.get_tot_prec3_var() != NULL)
+            else if (msg->get_tot_prec3_var() != NULL)
                 prec_code = WR_VAR(0, 13, 20);
-            else if (msg.get_tot_prec1_var() != NULL)
+            else if (msg->get_tot_prec1_var() != NULL)
                 prec_code = WR_VAR(0, 13, 19);
         }
         if (prec_code == 0)
@@ -188,26 +169,26 @@ struct SynopECMWF : public Synop
         bulletin.data_subcategory = 255;
         bulletin.data_subcategory_local = 1;
     }
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         Synop::to_subset(msg, subset);
         synop.add_ecmwf_synop_head();
         do_D01011();
         do_D01012();
         do_D01021();
-        /* 10 */ add(WR_VAR(0,  7,  1), DBA_MSG_HEIGHT_STATION);
+        /* 10 */ add(WR_VAR(0,  7,  1), sc::height_station);
     }
 };
 
 struct SynopECMWFLand : public SynopECMWF
 {
-    SynopECMWFLand(const Exporter::Options& opts, const Messages& msgs)
+    SynopECMWFLand(const dballe::ExporterOptions& opts, const Messages& msgs)
         : SynopECMWF(opts, msgs) {}
 
-    virtual const char* name() const { return SYNOP_ECMWF_LAND_NAME; }
-    virtual const char* description() const { return SYNOP_ECMWF_LAND_DESC; }
+    const char* name() const override { return SYNOP_ECMWF_LAND_NAME; }
+    const char* description() const override { return SYNOP_ECMWF_LAND_DESC; }
 
-    virtual void setupBulletin(wreport::Bulletin& bulletin)
+    void setupBulletin(wreport::Bulletin& bulletin) override
     {
         SynopECMWF::setupBulletin(bulletin);
 
@@ -230,36 +211,36 @@ struct SynopECMWFLand : public SynopECMWF
         bulletin.load_tables();
     }
 
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         SynopECMWF::to_subset(msg, subset);
         synop.add_D02001();
         synop.add_ecmwf_synop_weather();
-        /* 24 */ add(WR_VAR(0, 20, 10), DBA_MSG_CLOUD_N);
+        /* 24 */ add(WR_VAR(0, 20, 10), sc::cloud_n);
         /* 25 */ add(WR_VAR(0,  8,  2), WR_VAR(0, 8, 2), Level::cloud(258, 0), Trange::instant());
-        /* 26 */ add(WR_VAR(0, 20, 11), DBA_MSG_CLOUD_NH);
-        /* 27 */ add(WR_VAR(0, 20, 13), DBA_MSG_CLOUD_HH);
-        /* 28 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CL);
-        /* 29 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CM);
-        /* 30 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CH);
+        /* 26 */ add(WR_VAR(0, 20, 11), sc::cloud_nh);
+        /* 27 */ add(WR_VAR(0, 20, 13), sc::cloud_hh);
+        /* 28 */ add(WR_VAR(0, 20, 12), sc::cloud_cl);
+        /* 29 */ add(WR_VAR(0, 20, 12), sc::cloud_cm);
+        /* 30 */ add(WR_VAR(0, 20, 12), sc::cloud_ch);
         /* 31 */ add(WR_VAR(0,  8,  2), WR_VAR(0, 8, 2), Level::cloud(259, 1), Trange::instant());
-        /* 32 */ add(WR_VAR(0, 20, 11), DBA_MSG_CLOUD_N1);
-        /* 33 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_C1);
-        /* 34 */ add(WR_VAR(0, 20, 13), DBA_MSG_CLOUD_H1);
+        /* 32 */ add(WR_VAR(0, 20, 11), sc::cloud_n1);
+        /* 33 */ add(WR_VAR(0, 20, 12), sc::cloud_c1);
+        /* 34 */ add(WR_VAR(0, 20, 13), sc::cloud_h1);
         /* 35 */ add(WR_VAR(0,  8,  2), WR_VAR(0, 8, 2), Level::cloud(259, 2), Trange::instant());
-        /* 36 */ add(WR_VAR(0, 20, 11), DBA_MSG_CLOUD_N2);
-        /* 37 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_C2);
-        /* 38 */ add(WR_VAR(0, 20, 13), DBA_MSG_CLOUD_H2);
+        /* 36 */ add(WR_VAR(0, 20, 11), sc::cloud_n2);
+        /* 37 */ add(WR_VAR(0, 20, 12), sc::cloud_c2);
+        /* 38 */ add(WR_VAR(0, 20, 13), sc::cloud_h2);
         /* 39 */ add(WR_VAR(0,  8,  2), WR_VAR(0, 8, 2), Level::cloud(259, 3), Trange::instant());
-        /* 40 */ add(WR_VAR(0, 20, 11), DBA_MSG_CLOUD_N3);
-        /* 41 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_C3);
-        /* 42 */ add(WR_VAR(0, 20, 13), DBA_MSG_CLOUD_H3);
+        /* 40 */ add(WR_VAR(0, 20, 11), sc::cloud_n3);
+        /* 41 */ add(WR_VAR(0, 20, 12), sc::cloud_c3);
+        /* 42 */ add(WR_VAR(0, 20, 13), sc::cloud_h3);
         /* 43 */ add(WR_VAR(0,  8,  2), WR_VAR(0, 8, 2), Level::cloud(259, 4), Trange::instant());
-        /* 44 */ add(WR_VAR(0, 20, 11), DBA_MSG_CLOUD_N4);
-        /* 45 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_C4);
-        /* 46 */ add(WR_VAR(0, 20, 13), DBA_MSG_CLOUD_H4);
+        /* 44 */ add(WR_VAR(0, 20, 11), sc::cloud_n4);
+        /* 45 */ add(WR_VAR(0, 20, 12), sc::cloud_c4);
+        /* 46 */ add(WR_VAR(0, 20, 13), sc::cloud_h4);
         /* 47 */ add_prec();
-        /* 48 */ add(WR_VAR(0, 13, 13), DBA_MSG_TOT_SNOW);
+        /* 48 */ add(WR_VAR(0, 13, 13), sc::tot_snow);
 
         if (!is_crex)
         {
@@ -278,13 +259,13 @@ struct SynopECMWFLand : public SynopECMWF
 
 struct SynopECMWFLandHigh : public SynopECMWF
 {
-    SynopECMWFLandHigh(const Exporter::Options& opts, const Messages& msgs)
+    SynopECMWFLandHigh(const dballe::ExporterOptions& opts, const Messages& msgs)
         : SynopECMWF(opts, msgs) {}
 
-    virtual const char* name() const { return SYNOP_ECMWF_LAND_HIGH_NAME; }
-    virtual const char* description() const { return SYNOP_ECMWF_LAND_HIGH_DESC; }
+    const char* name() const override { return SYNOP_ECMWF_LAND_HIGH_NAME; }
+    const char* description() const override { return SYNOP_ECMWF_LAND_HIGH_DESC; }
 
-    virtual void setupBulletin(wreport::Bulletin& bulletin)
+    void setupBulletin(wreport::Bulletin& bulletin) override
     {
         SynopECMWF::setupBulletin(bulletin);
 
@@ -307,7 +288,7 @@ struct SynopECMWFLandHigh : public SynopECMWF
         bulletin.load_tables();
     }
 
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         SynopECMWF::to_subset(msg, subset);
 
@@ -317,15 +298,15 @@ struct SynopECMWFLandHigh : public SynopECMWF
         /* 14 */ add(WR_VAR(0, 10, 61), synop.v_pchange3);
         /* 15 */ add(WR_VAR(0, 10, 63), synop.v_ptend);
         synop.add_ecmwf_synop_weather();
-        /* 25 */ add(WR_VAR(0, 20, 10), DBA_MSG_CLOUD_N);
+        /* 25 */ add(WR_VAR(0, 20, 10), sc::cloud_n);
         /* 26 */ add(WR_VAR(0,  8,  2), WR_VAR(0, 8, 2), Level::cloud(258, 0), Trange::instant());
-        /* 27 */ add(WR_VAR(0, 20, 11), DBA_MSG_CLOUD_NH);
-        /* 28 */ add(WR_VAR(0, 20, 13), DBA_MSG_CLOUD_HH);
-        /* 29 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CL);
-        /* 30 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CM);
-        /* 31 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CH);
+        /* 27 */ add(WR_VAR(0, 20, 11), sc::cloud_nh);
+        /* 28 */ add(WR_VAR(0, 20, 13), sc::cloud_hh);
+        /* 29 */ add(WR_VAR(0, 20, 12), sc::cloud_cl);
+        /* 30 */ add(WR_VAR(0, 20, 12), sc::cloud_cm);
+        /* 31 */ add(WR_VAR(0, 20, 12), sc::cloud_ch);
         /* 32 */ add_prec();
-        /* 33 */ add(WR_VAR(0, 13, 13), DBA_MSG_TOT_SNOW);
+        /* 33 */ add(WR_VAR(0, 13, 13), sc::tot_snow);
 
         if (!is_crex)
         {
@@ -345,13 +326,13 @@ struct SynopECMWFLandHigh : public SynopECMWF
 // Same as SynopECMWFLandHigh but just with a different local subtype
 struct SynopECMWFAuto : public SynopECMWFLand
 {
-    SynopECMWFAuto(const Exporter::Options& opts, const Messages& msgs)
+    SynopECMWFAuto(const dballe::ExporterOptions& opts, const Messages& msgs)
         : SynopECMWFLand(opts, msgs) {}
 
-    virtual const char* name() const { return SYNOP_ECMWF_AUTO_NAME; }
-    virtual const char* description() const { return SYNOP_ECMWF_AUTO_DESC; }
+    const char* name() const override { return SYNOP_ECMWF_AUTO_NAME; }
+    const char* description() const override { return SYNOP_ECMWF_AUTO_DESC; }
 
-    virtual void setupBulletin(wreport::Bulletin& bulletin)
+    void setupBulletin(wreport::Bulletin& bulletin) override
     {
         SynopECMWFLand::setupBulletin(bulletin);
 
@@ -366,7 +347,7 @@ struct SynopWMO : public Synop
     // we process its subsets
     Bulletin* cur_bulletin;
 
-    SynopWMO(const Exporter::Options& opts, const Messages& msgs)
+    SynopWMO(const dballe::ExporterOptions& opts, const Messages& msgs)
         : Synop(opts, msgs), cur_bulletin(0) {}
 
     virtual const char* name() const { return SYNOP_WMO_NAME; }
@@ -394,10 +375,10 @@ struct SynopWMO : public Synop
     // vertical coordinates
     void do_D01090()
     {
-        add(WR_VAR(0,  1,  1), c_station);
-        add(WR_VAR(0,  1,  2), c_station);
+        add(WR_VAR(0,  1,  1), msg->station_data);
+        add(WR_VAR(0,  1,  2), msg->station_data);
         do_station_name(WR_VAR(0,  1, 15));
-        add(WR_VAR(0,  2,  1), c_station);
+        add(WR_VAR(0,  2,  1), msg->station_data);
         do_D01011();
         do_D01012();
         do_D01021();
@@ -405,7 +386,7 @@ struct SynopWMO : public Synop
     }
 
     // D02036  Clouds with bases below station level
-    void do_D02036(const Msg& msg, wreport::Subset& subset)
+    void do_D02036(const Message& msg, wreport::Subset& subset)
     {
         // Number of individual cloud layers or masses
         int max_cloud_group = 0;
@@ -430,19 +411,19 @@ struct SynopWMO : public Synop
     }
 
     // D02047  Direction of cloud drift
-    void do_D02047(const Msg& msg, wreport::Subset& subset)
+    void do_D02047(const Message& msg, wreport::Subset& subset)
     {
         // D02047  Direction of cloud drift
         for (int i = 1; i <= 3; ++i)
         {
             if (const msg::Context* c = msg.find_context(Level::cloud(260, i), Trange::instant()))
             {
-                if (const Var* var = c->find(WR_VAR(0,  8,  2)))
+                if (const Var* var = c->values.maybe_var(WR_VAR(0,  8,  2)))
                     subset.store_variable(*var);
                 else
                     subset.store_variable_undef(WR_VAR(0,  8,  2));
 
-                if (const Var* var = c->find(WR_VAR(0, 20, 54)))
+                if (const Var* var = c->values.maybe_var(WR_VAR(0, 20, 54)))
                     subset.store_variable(*var);
                 else
                     subset.store_variable_undef(WR_VAR(0, 20, 54));
@@ -454,21 +435,21 @@ struct SynopWMO : public Synop
     }
 
     // D02048  Direction and elevation of cloud
-    void do_D02048(const Msg& msg, wreport::Subset& subset)
+    void do_D02048(const Message& msg, wreport::Subset& subset)
     {
         if (const msg::Context* c = msg.find_context(Level::cloud(262, 0), Trange::instant()))
         {
-            if (const Var* var = c->find(WR_VAR(0,  5, 21)))
+            if (const Var* var = c->values.maybe_var(WR_VAR(0,  5, 21)))
                 subset.store_variable(*var);
             else
                 subset.store_variable_undef(WR_VAR(0,  5, 21));
 
-            if (const Var* var = c->find(WR_VAR(0,  7, 21)))
+            if (const Var* var = c->values.maybe_var(WR_VAR(0,  7, 21)))
                 subset.store_variable(*var);
             else
                 subset.store_variable_undef(WR_VAR(0,  7, 21));
 
-            if (const Var* var = c->find(WR_VAR(0, 20, 12)))
+            if (const Var* var = c->values.maybe_var(WR_VAR(0, 20, 12)))
                 subset.store_variable(*var);
             else
                 subset.store_variable_undef(WR_VAR(0, 20, 12));
@@ -482,11 +463,11 @@ struct SynopWMO : public Synop
     }
 
     // D02037  State of ground, snow depth, ground minimum temperature
-    void do_D02037(const Msg& msg, wreport::Subset& subset)
+    void do_D02037(const Message& msg, wreport::Subset& subset)
     {
-        add(WR_VAR(0, 20,  62), DBA_MSG_STATE_GROUND);
-        add(WR_VAR(0, 13,  13), DBA_MSG_TOT_SNOW);
-        if (const Var* var = msg.get(WR_VAR(0, 12, 121), Level(1), Trange(3, 0, 43200)))
+        add(WR_VAR(0, 20,  62), sc::state_ground);
+        add(WR_VAR(0, 13,  13), sc::tot_snow);
+        if (const Var* var = msg.get(Level(1), Trange(3, 0, 43200), WR_VAR(0, 12, 121)))
             subset.store_variable(WR_VAR(0, 12, 113), *var);
         else
             subset.store_variable_undef(WR_VAR(0, 12, 113));
@@ -501,7 +482,7 @@ struct SynopWMO : public Synop
         if (c_sunshine1)
         {
             subset->store_variable_d(WR_VAR(0,  4, 24), -c_sunshine1->trange.p2 / 3600);
-            if (const Var* var = c_sunshine1->find(WR_VAR(0, 14, 31)))
+            if (const Var* var = c_sunshine1->values.maybe_var(WR_VAR(0, 14, 31)))
                 subset->store_variable(*var);
             else
                 subset->store_variable_undef(WR_VAR(0, 14, 31));
@@ -512,7 +493,7 @@ struct SynopWMO : public Synop
         if (c_sunshine2)
         {
             subset->store_variable_d(WR_VAR(0,  4, 24), -c_sunshine2->trange.p2 / 3600);
-            if (const Var* var = c_sunshine2->find(WR_VAR(0, 14, 31)))
+            if (const Var* var = c_sunshine2->values.maybe_var(WR_VAR(0, 14, 31)))
                 subset->store_variable(*var);
             else
                 subset->store_variable_undef(WR_VAR(0, 14, 31));
@@ -532,7 +513,7 @@ struct SynopWMO : public Synop
         subset->store_variable_undef(WR_VAR(0,  7, 32));
     }
 
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    virtual void to_subset(const Message& msg, wreport::Subset& subset)
     {
         Synop::to_subset(msg, subset);
 
@@ -655,9 +636,9 @@ struct SynopWMO : public Synop
 void register_synop(TemplateRegistry& r)
 {
     r.register_factory(0, "synop", "Synop (autodetect)",
-            [](const Exporter::Options& opts, const Messages& msgs) {
-                const Msg& msg = Msg::downcast(msgs[0]);
-                const Var* var = msg.get_st_name_var();
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
+                auto msg = Message::downcast(msgs[0]);
+                const Var* var = msg->get_st_name_var();
                 if (var)
                 {
                     const wr::TemplateFactory& fac = wr::TemplateRegistry::get(SYNOP_WMO_NAME);
@@ -670,35 +651,34 @@ void register_synop(TemplateRegistry& r)
                 }
             });
     r.register_factory(0, SYNOP_WMO_NAME, SYNOP_WMO_DESC,
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new SynopWMO(opts, msgs));
             });
     r.register_factory(0, "synop-ecmwf", "Synop ECMWF (autodetect) (0.1)",
-            [](const Exporter::Options& opts, const Messages& msgs) {
-                const Msg& msg = Msg::downcast(msgs[0]);
-                const Var* var = msg.get_st_type_var();
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
+                auto msg = Message::downcast(msgs[0]);
+                const Var* var = msg->get_st_type_var();
                 if (var != NULL && var->enqi() == 0)
                     return unique_ptr<Template>(new SynopECMWFAuto(opts, msgs));
 
                 // If it has a geopotential, it's a land high station
-                for (std::vector<msg::Context*>::const_iterator i = msg.data.begin();
-                        i != msg.data.end(); ++i)
-                    if ((*i)->level.ltype1 == 100)
-                        if ((*i)->find(WR_VAR(0, 10, 8)))
+                for (const auto& ctx: msg->data)
+                    if (ctx.level.ltype1 == 100)
+                        if (ctx.values.maybe_var(WR_VAR(0, 10, 8)))
                             return unique_ptr<Template>(new SynopECMWFLandHigh(opts, msgs));
 
                 return unique_ptr<Template>(new SynopECMWFLand(opts, msgs));
             });
     r.register_factory(0, SYNOP_ECMWF_LAND_NAME, SYNOP_ECMWF_LAND_DESC,
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new SynopECMWFLand(opts, msgs));
             });
     r.register_factory(0, SYNOP_ECMWF_LAND_HIGH_NAME, SYNOP_ECMWF_LAND_HIGH_DESC,
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new SynopECMWFLandHigh(opts, msgs));
             });
     r.register_factory(0, SYNOP_ECMWF_AUTO_NAME, SYNOP_ECMWF_AUTO_DESC,
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new SynopECMWFAuto(opts, msgs));
             });
 }
@@ -706,3 +686,4 @@ void register_synop(TemplateRegistry& r)
 }
 }
 }
+}
