@@ -1,25 +1,7 @@
-/*
- * Copyright (C) 2005--2015  ARPA-SIM <urpsim@smr.arpa.emr.it>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
- *
- * Author: Enrico Zini <enrico@enricozini.com>
- */
-
-#include "msg/wr_codec.h"
-#include "msg/msg.h"
-#include "msg/context.h"
+#include "dballe/msg/wr_codec.h"
+#include "dballe/core/shortcuts.h"
+#include "dballe/msg/msg.h"
+#include "dballe/msg/context.h"
 #include <wreport/bulletin.h>
 #include <wreport/vartable.h>
 #include <wreport/conv.h>
@@ -49,6 +31,7 @@ using namespace std;
 
 
 namespace dballe {
+namespace impl {
 namespace msg {
 namespace wr {
 
@@ -59,16 +42,16 @@ struct TempBase : public Template
 {
     bool is_crex;
 
-    TempBase(const Exporter::Options& opts, const Messages& msgs)
+    TempBase(const dballe::ExporterOptions& opts, const Messages& msgs)
         : Template(opts, msgs) {}
 
     /// Count the number of sounding levels
     int count_levels() const
     {
         int lev_no = 0;
-        for (size_t i = 0; i < msg->data.size(); ++i)
+        for (const auto& ctx: msg->data)
         {
-            if (msg->data[i]->find_vsig() != NULL)
+            if (ctx.find_vsig() != NULL)
                 ++lev_no;
         }
         return lev_no;
@@ -80,9 +63,9 @@ struct TempBase : public Template
         subset->store_variable_i(WR_VAR(0, 31,  1), count);
 
         // Iterate backwards as we need to add levels in decreasing pressure order
-        for (int i = msg->data.size() - 1; i >= 0; --i)
+        for (auto i = msg->data.rbegin(); i != msg->data.rend(); ++i)
         {
-            msg::Context& c = *msg->data[i];
+            const msg::Context& c = *i;
 
             // We only want levels with a vertical sounding significance
             const Var* vss = c.find_vsig();
@@ -93,9 +76,9 @@ struct TempBase : public Template
             double press = c.level.l1;
 
             /* Add pressure */
-            const Var* press_var = c.find(WR_VAR(0, 10, 4));
+            const Var* press_var = c.values.maybe_var(WR_VAR(0, 10, 4));
             if (!press_var)
-                press_var = c.find(WR_VAR(0, 7, 4));
+                press_var = c.values.maybe_var(WR_VAR(0, 7, 4));
             if (press_var)
                 subset->store_variable(WR_VAR(0, 7, 4), *press_var);
             else if (press == MISSING_INT)
@@ -128,7 +111,7 @@ struct TempBase : public Template
         return count;
     }
 
-    virtual void setupBulletin(wreport::Bulletin& bulletin)
+    void setupBulletin(wreport::Bulletin& bulletin) override
     {
         Template::setupBulletin(bulletin);
 
@@ -147,21 +130,21 @@ struct TempBase : public Template
         bulletin.data_subcategory_local = 101;
         for (const auto& mi: msgs)
         {
-            const Msg& msg = Msg::downcast(mi);
-            if (msg.type == MSG_PILOT)
+            auto msg = Message::downcast(mi);
+            if (msg->type == MessageType::PILOT)
             {
                 bulletin.data_subcategory_local = 91;
                 break;
-            } else if (msg.get_ident_var()) {
+            } else if (msg->get_ident_var()) {
                 bulletin.data_subcategory_local = 102;
                 break;
-            } else if (msg.get_block_var()) {
+            } else if (msg->get_block_var()) {
                 break;
             }
         }
     }
 
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         Template::to_subset(msg, subset);
     }
@@ -199,13 +182,13 @@ struct TempBase : public Template
 
 struct TempWMO : public TempBase
 {
-    TempWMO(const Exporter::Options& opts, const Messages& msgs)
+    TempWMO(const dballe::ExporterOptions& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
-    virtual const char* name() const { return TEMP_WMO_NAME; }
-    virtual const char* description() const { return TEMP_WMO_DESC; }
+    const char* name() const override { return TEMP_WMO_NAME; }
+    const char* description() const override { return TEMP_WMO_DESC; }
 
-    virtual void setupBulletin(wreport::Bulletin& bulletin)
+    void setupBulletin(wreport::Bulletin& bulletin) override
     {
         TempBase::setupBulletin(bulletin);
 
@@ -214,8 +197,8 @@ struct TempWMO : public TempBase
         bulletin.data_subcategory_local = 255;
         for (const auto& mi : msgs)
         {
-            const Msg& msg = Msg::downcast(mi);
-            if (msg.type == MSG_TEMP_SHIP)
+            auto msg = Message::downcast(mi);
+            if (msg->type == MessageType::TEMP_SHIP)
             {
                 bulletin.data_subcategory = 5;
                 break;
@@ -231,7 +214,7 @@ struct TempWMO : public TempBase
     void do_D03054(const msg::Context& c)
     {
         add(WR_VAR(0,  4,  86), &c);
-        if (const Var* vss = c.find(WR_VAR(0, 8, 42)))
+        if (const Var* vss = c.values.maybe_var(WR_VAR(0, 8, 42)))
         {
             // Deal with 'missing VSS' group markers
             if (vss->enq((int)BUFR08042::MISSING) & BUFR08042::MISSING)
@@ -252,7 +235,7 @@ struct TempWMO : public TempBase
                 break;
             case 102:
                 add(WR_VAR(0, 7, 4), &c, WR_VAR(0, 10, 4));
-                if (const Var* var = c.find(WR_VAR(0, 10, 8)))
+                if (const Var* var = c.values.maybe_var(WR_VAR(0, 10, 8)))
                     add(WR_VAR(0, 10, 9), var);
                 else
                     subset->store_variable_d(WR_VAR(0, 10, 9), (double)c.level.l1);
@@ -270,33 +253,33 @@ struct TempWMO : public TempBase
         add(WR_VAR(0, 11,   2), &c);
     }
 
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         TempBase::to_subset(msg, subset);
         do_D01001(); // station id
-        add(WR_VAR(0,  1, 11), c_station, DBA_MSG_IDENT);
-        add(WR_VAR(0,  2, 11), c_gnd_instant, DBA_MSG_SONDE_TYPE);
-        add(WR_VAR(0,  2, 13), c_gnd_instant, DBA_MSG_SONDE_CORRECTION);
-        add(WR_VAR(0,  2, 14), c_gnd_instant, DBA_MSG_SONDE_TRACKING);
-        add(WR_VAR(0,  2,  3), c_gnd_instant, DBA_MSG_MEAS_EQUIP_TYPE);
+        add(WR_VAR(0,  1, 11), msg.station_data, sc::ident);
+        add(WR_VAR(0,  2, 11), c_gnd_instant, sc::sonde_type);
+        add(WR_VAR(0,  2, 13), c_gnd_instant, sc::sonde_correction);
+        add(WR_VAR(0,  2, 14), c_gnd_instant, sc::sonde_tracking);
+        add(WR_VAR(0,  2,  3), c_gnd_instant, sc::meas_equip_type);
         subset.store_variable_i(WR_VAR(0, 8, 21), 18);
         do_D01011(); // date
         do_D01013(); // time
         do_D01021(); // coordinates
-        add(WR_VAR(0,  7, 30), c_station, DBA_MSG_HEIGHT_STATION);
-        add(WR_VAR(0,  7, 31), c_station, DBA_MSG_HEIGHT_BARO);
-        add(WR_VAR(0,  7,  7), c_station, DBA_MSG_HEIGHT_RELEASE);
-        add(WR_VAR(0, 33, 24), c_station, DBA_MSG_STATION_HEIGHT_QUALITY);
+        add(WR_VAR(0,  7, 30), msg.station_data, sc::height_station);
+        add(WR_VAR(0,  7, 31), msg.station_data, sc::height_baro);
+        add(WR_VAR(0,  7,  7), msg.station_data, sc::height_release);
+        add(WR_VAR(0, 33, 24), msg.station_data, sc::station_height_quality);
 
         // Cloud information reported with vertical soundings
         add(WR_VAR(0,  8,  2), WR_VAR(0, 8, 2), Level::cloud(258, 0), Trange::instant());
-        add(WR_VAR(0, 20, 11), DBA_MSG_CLOUD_NH);
-        add(WR_VAR(0, 20, 13), DBA_MSG_CLOUD_HH);
-        add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CL);
-        add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CM);
-        add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CH);
+        add(WR_VAR(0, 20, 11), sc::cloud_nh);
+        add(WR_VAR(0, 20, 13), sc::cloud_hh);
+        add(WR_VAR(0, 20, 12), sc::cloud_cl);
+        add(WR_VAR(0, 20, 12), sc::cloud_cm);
+        add(WR_VAR(0, 20, 12), sc::cloud_ch);
         subset.store_variable_undef(WR_VAR(0, 8, 2));
-        add(WR_VAR(0, 22, 43), c_gnd_instant, DBA_MSG_WATER_TEMP);
+        add(WR_VAR(0, 22, 43), c_gnd_instant, sc::water_temp);
 
         // Undef for now, we fill it later
         size_t rep_count_pos = subset.size();
@@ -304,26 +287,25 @@ struct TempWMO : public TempBase
 
         // Temperature, dew-point and wind data at pressure levels
         int group_count = 0;
-        for (std::vector<msg::Context*>::const_reverse_iterator i = msg.data.rbegin();
-                i != msg.data.rend(); ++i)
+        for (auto i = msg.data.rbegin(); i != msg.data.rend(); ++i)
         {
             // Iterate backwards to get pressure levels sorted from the higher
             // to the lower pressure
-            const msg::Context* c = *i;
+            const msg::Context& c = *i;
             // Skip non-presure levels
-            if (!c->find_vsig()) continue;
+            if (!c.find_vsig()) continue;
             // Skip levels not for us
             if (!(
-                  c->find(WR_VAR(0, 10,   8))
-               || c->find(WR_VAR(0, 12, 101))
-               || c->find(WR_VAR(0, 12, 103))
-               || c->find(WR_VAR(0, 11,   1))
-               || c->find(WR_VAR(0, 11,   2))) && (
-                  c->find(WR_VAR(0, 11, 61))
-               || c->find(WR_VAR(0, 11, 62)))
+                  c.values.maybe_var(WR_VAR(0, 10,   8))
+               || c.values.maybe_var(WR_VAR(0, 12, 101))
+               || c.values.maybe_var(WR_VAR(0, 12, 103))
+               || c.values.maybe_var(WR_VAR(0, 11,   1))
+               || c.values.maybe_var(WR_VAR(0, 11,   2))) && (
+                  c.values.maybe_var(WR_VAR(0, 11, 61))
+               || c.values.maybe_var(WR_VAR(0, 11, 62)))
                )
                 continue;
-            do_D03054(*c);
+            do_D03054(c);
             ++group_count;
         }
         subset[rep_count_pos].seti(group_count);
@@ -332,20 +314,19 @@ struct TempWMO : public TempBase
         rep_count_pos = subset.size();
         subset.store_variable_undef(WR_VAR(0, 31, 1));
         group_count = 0;
-        for (std::vector<msg::Context*>::const_reverse_iterator i = msg.data.rbegin();
-                i != msg.data.rend(); ++i)
+        for (auto i = msg.data.rbegin(); i != msg.data.rend(); ++i)
         {
             // Iterate backwards to get pressure levels sorted from the higher
             // to the lower pressure
-            const msg::Context* c = *i;
+            const msg::Context& c = *i;
             // Skip non-pressure levels
-            if (c->level.ltype1 != 100) continue;
+            if (c.level.ltype1 != 100) continue;
             // Skip levels not for us
             if (!(
-                  c->find(WR_VAR(0, 11, 61))
-               || c->find(WR_VAR(0, 11, 62))))
+                  c.values.maybe_var(WR_VAR(0, 11, 61))
+               || c.values.maybe_var(WR_VAR(0, 11, 62))))
                 continue;
-            if (do_D03051(*c))
+            if (do_D03051(c))
                 ++group_count;
         }
         subset[rep_count_pos].seti(group_count);
@@ -354,7 +335,7 @@ struct TempWMO : public TempBase
 
 struct TempRadar : public TempBase
 {
-    TempRadar(const Exporter::Options& opts, const Messages& msgs)
+    TempRadar(const dballe::ExporterOptions& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return TEMP_RADAR_NAME; }
@@ -391,16 +372,16 @@ struct TempRadar : public TempBase
         bulletin.load_tables();
     }
 
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         TempBase::to_subset(msg, subset);
-        add(WR_VAR(0, 1, 33), c_station);
-        add(WR_VAR(0, 1, 34), c_station);
+        add(WR_VAR(0, 1, 33), msg.station_data);
+        add(WR_VAR(0, 1, 34), msg.station_data);
         do_D01001(); // station id
         do_D01011(); // date
         do_D01012(); // date
         do_D01022(); // date
-        add(WR_VAR(0,  2,  3), c_gnd_instant, DBA_MSG_MEAS_EQUIP_TYPE);
+        add(WR_VAR(0,  2,  3), c_gnd_instant, sc::meas_equip_type);
         add(WR_VAR(0, 2, 121), c_gnd_instant);
         add(WR_VAR(0, 2, 125), c_gnd_instant);
 
@@ -410,24 +391,23 @@ struct TempRadar : public TempBase
 
         // Sounding levels
         int group_count = 0;
-        for (std::vector<msg::Context*>::const_reverse_iterator i = msg.data.rbegin();
-                i != msg.data.rend(); ++i)
+        for (auto i = msg.data.rbegin(); i != msg.data.rend(); ++i)
         {
             // Iterate backwards to get pressure levels sorted from the higher
             // to the lower pressure
-            const msg::Context* c = *i;
+            const msg::Context& c = *i;
             // We are only interested in height levels
-            if (c->level.ltype1 != 102) continue;
-            if (const Var* var = c->find(WR_VAR(0, 7, 7)))
+            if (c.level.ltype1 != 102) continue;
+            if (const Var* var = c.values.maybe_var(WR_VAR(0, 7, 7)))
                 add(WR_VAR(0, 7, 7), var);
             else
-                subset.store_variable_d(WR_VAR(0, 7, 7), c->level.l1/1000.0);
-            add(WR_VAR(0, 11,   1), c);
+                subset.store_variable_d(WR_VAR(0, 7, 7), c.level.l1/1000.0);
+            add(WR_VAR(0, 11,   1), &c);
             static const Varcode codes[] = { WR_VAR(0, 11,   2), WR_VAR(0, 11,   6) };
             for (unsigned i = 0; i < 2; ++i)
             {
                 Varcode code = codes[i];
-                if (const wreport::Var *var = c->find(code))
+                if (const wreport::Var *var = c.values.maybe_var(code))
                 {
                     add(code, var);
                     add(WR_VAR(0, 33, 2), var->enqa(WR_VAR(0, 33,  2)));
@@ -436,7 +416,7 @@ struct TempRadar : public TempBase
                     subset.store_variable_undef(WR_VAR(0, 33,   2));
                 }
             }
-            add(WR_VAR(0, 11,  50), c);
+            add(WR_VAR(0, 11,  50), &c);
             ++group_count;
         }
         subset[rep_count_pos].seti(group_count);
@@ -445,7 +425,7 @@ struct TempRadar : public TempBase
 
 struct TempEcmwfLand : public TempBase
 {
-    TempEcmwfLand(const Exporter::Options& opts, const Messages& msgs)
+    TempEcmwfLand(const dballe::ExporterOptions& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return TEMP_ECMWF_LAND_NAME; }
@@ -474,25 +454,25 @@ struct TempEcmwfLand : public TempBase
         bulletin.load_tables();
     }
 
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         TempBase::to_subset(msg, subset);
-        /*  0 */ add(WR_VAR(0,  1,  1), DBA_MSG_BLOCK);
-        /*  1 */ add(WR_VAR(0,  1,  2), DBA_MSG_STATION);
-        /*  2 */ add(WR_VAR(0,  2, 11), DBA_MSG_SONDE_TYPE);
-        /*  3 */ add(WR_VAR(0,  2, 12), DBA_MSG_SONDE_METHOD);
+        /*  0 */ add(WR_VAR(0,  1,  1), sc::block);
+        /*  1 */ add(WR_VAR(0,  1,  2), sc::station);
+        /*  2 */ add(WR_VAR(0,  2, 11), sc::sonde_type);
+        /*  3 */ add(WR_VAR(0,  2, 12), sc::sonde_method);
         do_D01011();
         do_D01012();
-        /*  9 */ add(WR_VAR(0,  5,  1), DBA_MSG_LATITUDE);
-        /* 10 */ add(WR_VAR(0,  6,  1), DBA_MSG_LONGITUDE);
-        /* 11 */ add(WR_VAR(0,  7,  1), DBA_MSG_HEIGHT_STATION);
-        /* 12 */ add(WR_VAR(0, 20, 10), DBA_MSG_CLOUD_N);
+        /*  9 */ add(WR_VAR(0,  5,  1), sc::latitude);
+        /* 10 */ add(WR_VAR(0,  6,  1), sc::longitude);
+        /* 11 */ add(WR_VAR(0,  7,  1), sc::height_station);
+        /* 12 */ add(WR_VAR(0, 20, 10), sc::cloud_n);
         /* 13 */ add(WR_VAR(0,  8,  2), WR_VAR(0, 8, 2), Level::cloud(258, 0), Trange::instant());
-        /* 14 */ add(WR_VAR(0, 20, 11), DBA_MSG_CLOUD_NH);
-        /* 15 */ add(WR_VAR(0, 20, 13), DBA_MSG_CLOUD_HH);
-        /* 16 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CL);
-        /* 17 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CM);
-        /* 18 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CH);
+        /* 14 */ add(WR_VAR(0, 20, 11), sc::cloud_nh);
+        /* 15 */ add(WR_VAR(0, 20, 13), sc::cloud_hh);
+        /* 16 */ add(WR_VAR(0, 20, 12), sc::cloud_cl);
+        /* 17 */ add(WR_VAR(0, 20, 12), sc::cloud_cm);
+        /* 18 */ add(WR_VAR(0, 20, 12), sc::cloud_ch);
         add_sounding_levels();
         if (!is_crex)
         {
@@ -512,7 +492,7 @@ struct TempEcmwfLand : public TempBase
 
 struct TempEcmwfShip : public TempBase
 {
-    TempEcmwfShip(const Exporter::Options& opts, const Messages& msgs)
+    TempEcmwfShip(const dballe::ExporterOptions& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return TEMP_ECMWF_SHIP_NAME; }
@@ -556,26 +536,26 @@ struct TempEcmwfShip : public TempBase
         bulletin.load_tables();
     }
 
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         TempBase::to_subset(msg, subset);
-        /*  0 */ add(WR_VAR(0,  1, 11), DBA_MSG_IDENT);
-        /*  1 */ add(WR_VAR(0,  1, 12), DBA_MSG_ST_DIR);
-        /*  2 */ add(WR_VAR(0,  1, 13), DBA_MSG_ST_SPEED);
-        /*  3 */ add(WR_VAR(0,  2, 11), DBA_MSG_SONDE_TYPE);
-        /*  4 */ add(WR_VAR(0,  2, 12), DBA_MSG_SONDE_METHOD);
+        /*  0 */ add(WR_VAR(0,  1, 11), sc::ident);
+        /*  1 */ add(WR_VAR(0,  1, 12), sc::st_dir);
+        /*  2 */ add(WR_VAR(0,  1, 13), sc::st_speed);
+        /*  3 */ add(WR_VAR(0,  2, 11), sc::sonde_type);
+        /*  4 */ add(WR_VAR(0,  2, 12), sc::sonde_method);
         do_D01011();
         do_D01012();
-        /* 10 */ add(WR_VAR(0,  5,  2), DBA_MSG_LATITUDE);
-        /* 11 */ add(WR_VAR(0,  6,  2), DBA_MSG_LONGITUDE);
-        /* 12 */ add(WR_VAR(0,  7,  1), DBA_MSG_HEIGHT_STATION);
-        /* 13 */ add(WR_VAR(0, 20, 10), DBA_MSG_CLOUD_N);
+        /* 10 */ add(WR_VAR(0,  5,  2), sc::latitude);
+        /* 11 */ add(WR_VAR(0,  6,  2), sc::longitude);
+        /* 12 */ add(WR_VAR(0,  7,  1), sc::height_station);
+        /* 13 */ add(WR_VAR(0, 20, 10), sc::cloud_n);
         /* 14 */ add(WR_VAR(0,  8,  2), WR_VAR(0, 8, 2), Level::cloud(258, 0), Trange::instant());
-        /* 15 */ add(WR_VAR(0, 20, 11), DBA_MSG_CLOUD_NH);
-        /* 16 */ add(WR_VAR(0, 20, 13), DBA_MSG_CLOUD_HH);
-        /* 17 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CL);
-        /* 18 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CM);
-        /* 19 */ add(WR_VAR(0, 20, 12), DBA_MSG_CLOUD_CH);
+        /* 15 */ add(WR_VAR(0, 20, 11), sc::cloud_nh);
+        /* 16 */ add(WR_VAR(0, 20, 13), sc::cloud_hh);
+        /* 17 */ add(WR_VAR(0, 20, 12), sc::cloud_cl);
+        /* 18 */ add(WR_VAR(0, 20, 12), sc::cloud_cm);
+        /* 19 */ add(WR_VAR(0, 20, 12), sc::cloud_ch);
         add_sounding_levels();
         if (!is_crex)
         {
@@ -597,7 +577,7 @@ struct PilotWMO : public TempBase
 {
     bool pressure_levs;
 
-    PilotWMO(const Exporter::Options& opts, const Messages& msgs)
+    PilotWMO(const dballe::ExporterOptions& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return PILOT_WMO_NAME; }
@@ -611,11 +591,10 @@ struct PilotWMO : public TempBase
 
         for (const auto& mi: msgs)
         {
-            const Msg& msg = Msg::downcast(mi);
-            for (std::vector<msg::Context*>::const_iterator i = msg.data.begin();
-                    i != msg.data.end(); ++i)
+            auto msg = Message::downcast(mi);
+            for (const auto& ctx: msg->data)
             {
-                switch ((*i)->level.ltype1)
+                switch (ctx.level.ltype1)
                 {
                     case 100: ++has_press; break;
                     case 102: ++has_height; break;
@@ -630,7 +609,7 @@ struct PilotWMO : public TempBase
         TempBase::to_bulletin(bulletin);
     }
 
-    virtual void setupBulletin(wreport::Bulletin& bulletin)
+    void setupBulletin(wreport::Bulletin& bulletin) override
     {
         TempBase::setupBulletin(bulletin);
 
@@ -646,30 +625,31 @@ struct PilotWMO : public TempBase
             bulletin.datadesc.push_back(WR_VAR(3, 9, 51)); // For height levels
         bulletin.load_tables();
     }
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         TempBase::to_subset(msg, subset);
 
         // Identification of launch site and instrumentation
-        add(WR_VAR(0,  1,  1), DBA_MSG_BLOCK);
-        add(WR_VAR(0,  1,  2), DBA_MSG_STATION);
-        add(WR_VAR(0,  1, 11), DBA_MSG_IDENT);
-        add(WR_VAR(0,  2, 11), DBA_MSG_SONDE_TYPE);
-        add(WR_VAR(0,  2, 14), DBA_MSG_SONDE_TRACKING);
-        add(WR_VAR(0,  2,  3), DBA_MSG_MEAS_EQUIP_TYPE);
+        add(WR_VAR(0,  1,  1), sc::block);
+        add(WR_VAR(0,  1,  2), sc::station);
+        add(WR_VAR(0,  1, 11), sc::ident);
+        add(WR_VAR(0,  2, 11), sc::sonde_type);
+        add(WR_VAR(0,  2, 14), sc::sonde_tracking);
+        add(WR_VAR(0,  2,  3), sc::meas_equip_type);
 
         // Date/time of launch
-        add(WR_VAR(0,  8, 21), 18); // Launch time
+        add(WR_VAR(0,  8, 21), sc::timesig); // Launch time
         do_D01011();
         do_D01013();
 
         // Horizontal and vertical coordinates of launch site
-        add(WR_VAR(0,  5,  1), DBA_MSG_LATITUDE);
-        add(WR_VAR(0,  6,  1), DBA_MSG_LONGITUDE);
-        add(WR_VAR(0,  7, 30), DBA_MSG_HEIGHT_STATION);
-        add(WR_VAR(0,  7, 31), DBA_MSG_HEIGHT_BARO);
-        add(WR_VAR(0,  7,  7), DBA_MSG_HEIGHT_RELEASE);
-        add(WR_VAR(0, 33, 24), DBA_MSG_STATION_HEIGHT_QUALITY);
+        add(WR_VAR(0,  5,  1), sc::latitude);
+        add(WR_VAR(0,  6,  1), sc::longitude);
+        add(WR_VAR(0,  7, 30), sc::height_station);
+        add(WR_VAR(0,  7, 31), sc::height_baro);
+        add(WR_VAR(0,  7,  7), sc::height_release);
+        add(WR_VAR(0, 33, 24), sc::station_height_quality);
 
         // Keep track of where we stored
         int rep_count_pos = subset.size();
@@ -677,10 +657,9 @@ struct PilotWMO : public TempBase
 
         /* Iterate backwards as we need to add levels in decreasing pressure order */
         int group_count = 0;
-        for (std::vector<msg::Context*>::const_reverse_iterator i = msg.data.rbegin();
-                i != msg.data.rend(); ++i)
+        for (auto i = msg.data.rbegin(); i != msg.data.rend(); ++i)
         {
-            const msg::Context& c = **i;
+            const msg::Context& c = *i;
             // We only want levels with a vertical sounding significance
             const Var* vss = c.find_vsig();
             if (vss == NULL) continue;
@@ -715,12 +694,11 @@ struct PilotWMO : public TempBase
         rep_count_pos = subset.size();
         subset.store_variable_undef(WR_VAR(0, 31,  1));
         group_count = 0;
-        for (std::vector<msg::Context*>::const_reverse_iterator i = msg.data.rbegin();
-                i != msg.data.rend(); ++i)
+        for (auto i = msg.data.rbegin(); i != msg.data.rend(); ++i)
         {
             // Iterate backwards to get pressure levels sorted from the higher
             // to the lower pressure
-            const msg::Context& c = **i;
+            const msg::Context& c = *i;
             // Skip levels that do not fit
             if (pressure_levs && c.level.ltype1 != 100) continue;
             if (!pressure_levs && c.level.ltype1 != 102) continue;
@@ -739,7 +717,7 @@ struct PilotWMO : public TempBase
 
 struct PilotEcmwf : public TempBase
 {
-    PilotEcmwf(const Exporter::Options& opts, const Messages& msgs)
+    PilotEcmwf(const dballe::ExporterOptions& opts, const Messages& msgs)
         : TempBase(opts, msgs) {}
 
     virtual const char* name() const { return PILOT_ECMWF_NAME; }
@@ -779,34 +757,34 @@ struct PilotEcmwf : public TempBase
 
         bulletin.load_tables();
     }
-    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    void to_subset(const Message& msg, wreport::Subset& subset) override
     {
         TempBase::to_subset(msg, subset);
 
-        /*  0 */ add(WR_VAR(0,  1,  1), DBA_MSG_BLOCK);
-        /*  1 */ add(WR_VAR(0,  1,  2), DBA_MSG_STATION);
-        /*  2 */ add(WR_VAR(0,  2, 11), DBA_MSG_SONDE_TYPE);
-        /*  3 */ add(WR_VAR(0,  2, 12), DBA_MSG_SONDE_METHOD);
+        /*  0 */ add(WR_VAR(0,  1,  1), sc::block);
+        /*  1 */ add(WR_VAR(0,  1,  2), sc::station);
+        /*  2 */ add(WR_VAR(0,  2, 11), sc::sonde_type);
+        /*  3 */ add(WR_VAR(0,  2, 12), sc::sonde_method);
         do_D01011();
         do_D01012();
-        /*  9 */ add(WR_VAR(0,  5,  1), DBA_MSG_LATITUDE);
-        /* 10 */ add(WR_VAR(0,  6,  1), DBA_MSG_LONGITUDE);
-        /* 11 */ add(WR_VAR(0,  7,  1), DBA_MSG_HEIGHT_STATION);
+        /*  9 */ add(WR_VAR(0,  5,  1), sc::latitude);
+        /* 10 */ add(WR_VAR(0,  6,  1), sc::longitude);
+        /* 11 */ add(WR_VAR(0,  7,  1), sc::height_station);
 
         int rep_count_pos = subset.size();
         subset.store_variable_undef(WR_VAR(0, 31,  1));
 
         /* Iterate backwards as we need to add levels in decreasing pressure order */
         int group_count = 0;
-        for (int i = msg.data.size() - 1; i >= 0; --i)
+        for (auto i = msg.data.rbegin(); i != msg.data.rend(); ++i)
         {
-            msg::Context& c = *msg.data[i];
+            const msg::Context& c = *i;
             // We only want levels with a vertical sounding significance
             const Var* vss = c.find_vsig();
             if (vss == NULL) continue;
 
             /* Add pressure */
-            if (const Var* var = c.find(WR_VAR(0, 10, 4)))
+            if (const Var* var = c.values.maybe_var(WR_VAR(0, 10, 4)))
                 subset.store_variable(WR_VAR(0,  7,  4), *var);
             else if (c.level.ltype1 == 100)
                 subset.store_variable_d(WR_VAR(0,  7,  4), c.level.l1);
@@ -821,9 +799,9 @@ struct PilotEcmwf : public TempBase
             }
 
             /* Add geopotential */
-            if (const Var* var = c.find(WR_VAR(0, 10, 3)))
+            if (const Var* var = c.values.maybe_var(WR_VAR(0, 10, 3)))
                 subset.store_variable(WR_VAR(0, 10, 3), *var);
-            else if (const Var* var = c.find(WR_VAR(0, 10, 8)))
+            else if (const Var* var = c.values.maybe_var(WR_VAR(0, 10, 8)))
                 subset.store_variable(WR_VAR(0, 10, 3), *var);
             else if (c.level.ltype1 == 102)
                 subset.store_variable_d(WR_VAR(0, 10, 3), (double)c.level.l1 * 9.80665);
@@ -831,13 +809,13 @@ struct PilotEcmwf : public TempBase
                 subset.store_variable_undef(WR_VAR(0, 10, 3));
 
             /* Add wind direction */
-            if (const Var* var = c.find(WR_VAR(0, 11, 1)))
+            if (const Var* var = c.values.maybe_var(WR_VAR(0, 11, 1)))
                 subset.store_variable(WR_VAR(0, 11, 1), *var);
             else
                 subset.store_variable_undef(WR_VAR(0, 11, 1));
 
                 /* Add wind speed */
-            if (const Var* var = c.find(WR_VAR(0, 11, 2)))
+            if (const Var* var = c.values.maybe_var(WR_VAR(0, 11, 2)))
                 subset.store_variable(WR_VAR(0, 11, 2), *var);
             else
                 subset.store_variable_undef(WR_VAR(0, 11, 2));
@@ -869,19 +847,19 @@ struct PilotEcmwf : public TempBase
 void register_temp(TemplateRegistry& r)
 {
     r.register_factory(2, "temp", "Temp (autodetect)",
-            [](const Exporter::Options& opts, const Messages& msgs) {
-                const Msg& msg = Msg::downcast(msgs[0]);
-
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 // Get the type of equipment used
-                if (const wreport::Var* var = msg.get(WR_VAR(0, 2, 3), Level(1), Trange::instant()))
+                if (const wreport::Var* var = msgs[0]->get(Level(1), Trange::instant(), WR_VAR(0, 2, 3)))
                     // Is it a Radar?
                     if (var->enq(0) == 3)
                         return unique_ptr<Template>(new TempRadar(opts, msgs));
 
+                auto msg = Message::downcast(msgs[0]);
+
                 // ECMWF temps use normal replication which cannot do more than 256 levels
-                if (msg.data.size() > 260)
+                if (msg->data.size() > 260)
                     return unique_ptr<Template>(new TempWMO(opts, msgs));
-                const Var* var = msg.get_sonde_tracking_var();
+                const Var* var = msg->get_sonde_tracking_var();
                 if (var)
                     return unique_ptr<Template>(new TempWMO(opts, msgs));
                 else
@@ -891,46 +869,46 @@ void register_temp(TemplateRegistry& r)
                 }
             });
     r.register_factory(2, "temp-ship", "Temp ship (autodetect)",
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new TempEcmwfShip(opts, msgs));
             });
     r.register_factory(2, TEMP_WMO_NAME, TEMP_WMO_DESC,
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new TempWMO(opts, msgs));
             });
     r.register_factory(2, "temp-ecmwf", "Temp ECMWF (autodetect)",
-            [](const Exporter::Options& opts, const Messages& msgs) {
-                if (msgs.empty() || Msg::downcast(msgs[0]).type != MSG_TEMP_SHIP)
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
+                if (msgs.empty() || Message::downcast(msgs[0])->type != MessageType::TEMP_SHIP)
                     return unique_ptr<Template>(new TempEcmwfLand(opts, msgs));
                 else
                     return unique_ptr<Template>(new TempEcmwfShip(opts, msgs));
             });
     r.register_factory(2, TEMP_ECMWF_LAND_NAME, TEMP_ECMWF_LAND_DESC,
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new TempEcmwfLand(opts, msgs));
             });
     r.register_factory(2, TEMP_ECMWF_SHIP_NAME, TEMP_ECMWF_SHIP_DESC,
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new TempEcmwfShip(opts, msgs));
             });
     r.register_factory(6, TEMP_RADAR_NAME, TEMP_RADAR_DESC,
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new TempRadar(opts, msgs));
             });
     r.register_factory(2, PILOT_WMO_NAME, PILOT_WMO_DESC,
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new PilotWMO(opts, msgs));
             });
     r.register_factory(2, PILOT_ECMWF_NAME, PILOT_ECMWF_DESC,
-            [](const Exporter::Options& opts, const Messages& msgs) {
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
                 return unique_ptr<Template>(new PilotEcmwf(opts, msgs));
             });
     r.register_factory(2, "pilot", "pilot (autodetect)",
-            [](const Exporter::Options& opts, const Messages& msgs) {
-                const Msg& msg = Msg::downcast(msgs[0]);
-                const Var* var = msg.get_sonde_tracking_var();
+            [](const dballe::ExporterOptions& opts, const Messages& msgs) {
+                auto msg = Message::downcast(msgs[0]);
+                const Var* var = msg->get_sonde_tracking_var();
                 // Try with another one in case the first was just unset
-                if (!var) var = msg.get_meas_equip_type_var();
+                if (!var) var = msg->get_meas_equip_type_var();
                 if (var)
                     return unique_ptr<Template>(new PilotWMO(opts, msgs));
                 else
@@ -941,3 +919,4 @@ void register_temp(TemplateRegistry& r)
 }
 }
 }
+}
