@@ -5,17 +5,19 @@
  * Common base types used by most of DB-All.e code.
  */
 
+#include <dballe/fwd.h>
+#include <wreport/varinfo.h>
+#include <memory>
 #include <iosfwd>
-#include <limits.h>
+#include <functional>
+
+namespace wreport {
+class Var;
+}
 
 namespace dballe {
 struct CSVWriter;
 
-/**
- * Value to use for missing parts of level and time range values
- */
-static constexpr int MISSING_INT = INT_MAX;
-
 /**
  * Calendar date.
  *
@@ -85,6 +87,8 @@ struct Date
     static void julian_to_calendar(int jday, unsigned short& year, unsigned char& month, unsigned char& day);
 };
 
+std::ostream& operator<<(std::ostream& out, const Date& dt);
+
 
 /**
  * Time of the day.
@@ -148,6 +152,8 @@ struct Time
     static void validate(int ho, int mi, int se);
 };
 
+std::ostream& operator<<(std::ostream& out, const Time& t);
+
 
 /**
  * Date and time
@@ -191,6 +197,12 @@ struct Datetime
      */
     static Datetime upper_bound(int ye, int mo, int da, int ho, int mi, int se);
 
+    /// Fill possibly missing fields with their lowest valid value
+    void set_lower_bound();
+
+    /// Fill possibly missing fields with their highest valid value
+    void set_upper_bound();
+
     /// Return a Date with this date
     Date date() const;
 
@@ -212,18 +224,23 @@ struct Datetime
      */
     int compare(const Datetime& other) const;
 
-    bool operator==(const Datetime& dt) const;
-    bool operator!=(const Datetime& dt) const;
-    bool operator<(const Datetime& dt) const;
-    bool operator>(const Datetime& dt) const;
-    bool operator<=(const Datetime& dt) const;
-    bool operator>=(const Datetime& dt) const;
+    bool operator==(const Datetime& o) const;
+    bool operator!=(const Datetime& o) const;
+    bool operator<(const Datetime& o) const;
+    bool operator>(const Datetime& o) const;
+    bool operator<=(const Datetime& o) const;
+    bool operator>=(const Datetime& o) const;
 
     /**
      * Print to an output stream in ISO8601 combined format.
      */
     int print_iso8601(FILE* out, char sep='T', const char* end="\n") const;
 
+    /**
+     * Print to an output stream in ISO8601 combined format.
+     */
+    int print(FILE* out, const char* end="\n") const;
+
     /**
      * Write the datetime to an output stream in ISO8601 combined format.
      *
@@ -238,6 +255,9 @@ struct Datetime
      */
     void to_csv_iso8601(CSVWriter& out, char sep='T', const char* tz="") const;
 
+    /// Write to a string in ISO8601 combined format
+    std::string to_string(char sep='T', const char* tz="") const;
+
     /**
      * Parse an ISO8601 datetime string.
      *
@@ -262,6 +282,8 @@ struct Datetime
     static void normalise_h24(int& ye, int& mo, int& da, int& ho, int& mi, int& se);
 };
 
+std::ostream& operator<<(std::ostream& out, const Datetime& dt);
+
 
 /**
  * Range of datetimes.
@@ -285,8 +307,12 @@ struct DatetimeRange
     /// Check if this range is open on both sides
     bool is_missing() const;
 
-    bool operator==(const DatetimeRange& dtr) const;
-    bool operator!=(const DatetimeRange& dtr) const;
+    bool operator==(const DatetimeRange& o) const;
+    bool operator!=(const DatetimeRange& o) const;
+    bool operator<(const DatetimeRange& o) const;
+    bool operator<=(const DatetimeRange& o) const;
+    bool operator>(const DatetimeRange& o) const;
+    bool operator>=(const DatetimeRange& o) const;
 
     /// Set the extremes
     void set(const Datetime& min, const Datetime& max);
@@ -320,8 +346,13 @@ struct DatetimeRange
 
     /// Check if the two ranges are completely disjoint
     bool is_disjoint(const DatetimeRange& dtr) const;
+
+    /// Print to an output stream in ISO8601 combined format.
+    int print(FILE* out, const char* end="\n") const;
 };
 
+std::ostream& operator<<(std::ostream& out, const DatetimeRange& dtr);
+
 
 /**
  * Coordinates
@@ -337,16 +368,16 @@ struct DatetimeRange
 struct Coords
 {
     /// Latitude in 1/100000 of a degree (5 significant digits preserved)
-    int lat;
+    int lat = MISSING_INT;
 
     /**
      * Longitude in 1/100000 of a degree (5 significant digits preserved) and
      * normalised between -180.0 and 180.0.
      */
-    int lon;
+    int lon = MISSING_INT;
 
     /// Construct a missing Coords
-    Coords();
+    Coords() = default;
     /// Construct a coords from integers in 1/100000 of a degree
     Coords(int lat, int lon);
     /// Construct a coords from values in degrees
@@ -355,6 +386,18 @@ struct Coords
     /// Check if these coordinates are undefined
     bool is_missing() const;
 
+    /// Set the latitude only
+    void set_lat(double lat);
+
+    /// Set the longitude only
+    void set_lon(double lon);
+
+    /// Set the latitude only, in 1/100000 of a degree
+    void set_lat(int lat);
+
+    /// Set the longitude only, in 1/100000 of a degree
+    void set_lon(int lon);
+
     /// Set from integers in 1/100000 of a degree
     void set(int lat, int lon);
 
@@ -379,17 +422,34 @@ struct Coords
      */
     int compare(const Coords& o) const;
 
-    bool operator==(const Coords& dt) const;
-    bool operator!=(const Coords& dt) const;
-    bool operator<(const Coords& dt) const;
-    bool operator>(const Coords& dt) const;
-    bool operator<=(const Coords& dt) const;
-    bool operator>=(const Coords& dt) const;
+    bool operator==(const Coords& o) const;
+    bool operator!=(const Coords& o) const;
+    bool operator<(const Coords& o) const;
+    bool operator>(const Coords& o) const;
+    bool operator<=(const Coords& o) const;
+    bool operator>=(const Coords& o) const;
 
     /// Print to an output stream
     int print(FILE* out, const char* end="\n") const;
+
+    /// Format to a string
+    std::string to_string(const char* undef="-") const;
+
+    /// Convert a latitude to the internal integer representation
+    static int lat_to_int(double lat);
+
+    /// Convert a longitude to the internal integer representation
+    static int lon_to_int(double lat);
+
+    /// Convert a latitude from the internal integer representation
+    static double lat_from_int(int lat);
+
+    /// Convert a longitude from the internal integer representation
+    static double lon_from_int(int lon);
 };
 
+std::ostream& operator<<(std::ostream&, const Coords&);
+
 
 /**
  * Range of latitudes.
@@ -456,8 +516,18 @@ struct LatRange
 
     /// Check if a range is inside this range (extremes included)
     bool contains(const LatRange& lr) const;
+
+    /**
+     * Print the LatRange to a FILE*.
+     *
+     * @param out  The output stream
+     * @param end  String to print after the Station
+     */
+    int print(FILE* out, const char* end="\n") const;
 };
 
+std::ostream& operator<<(std::ostream& out, const LatRange& lr);
+
 
 /**
  * Range of longitudes.
@@ -511,12 +581,24 @@ struct LonRange
      */
     void get(double& min, double& max) const;
 
-    /// Set the extremes as integers
+    /**
+     * Set the extremes as integers, throwing an error in case of open ended
+     * ranges
+     */
     void set(int min, int max);
 
-    /// Set the extremes in degrees
+    /**
+     * Set the extremes in degrees, throwing an error in case of open ended
+     * ranges
+     */
     void set(double min, double max);
 
+    /**
+     * Set from another LonRange, throwing an error in case of open ended
+     * ranges
+     */
+    void set(const LonRange& lr);
+
     /// Check if a point is inside this range (extremes included)
     bool contains(int lon) const;
 
@@ -525,19 +607,29 @@ struct LonRange
 
     /// Check if a range is inside this range (extremes included)
     bool contains(const LonRange& lr) const;
+
+    /**
+     * Print the LonRange to a FILE*.
+     *
+     * @param out  The output stream
+     * @param end  String to print after the Station
+     */
+    int print(FILE* out, const char* end="\n") const;
 };
 
+std::ostream& operator<<(std::ostream& out, const LonRange& lr);
+
 
 /// Vertical level or layer
 struct Level
 {
-    /// Type of the level or the first layer. See @ref level_table.
+    /// Type of the level or the first layer
     int ltype1;
-    /// L1 value of the level or the first layer. See @ref level_table.
+    /// L1 value of the level or the first layer
     int l1;
-    /// Type of the the second layer. See @ref level_table.
+    /// Type of the the second layer
     int ltype2;
-    /// L2 value of the second layer. See @ref level_table.
+    /// L2 value of the second layer
     int l2;
 
     Level(int ltype1=MISSING_INT, int l1=MISSING_INT, int ltype2=MISSING_INT, int l2=MISSING_INT)
@@ -546,10 +638,12 @@ struct Level
     /// Check if this level is fully set to the missing value
     bool is_missing() const;
 
-    bool operator==(const Level& l) const;
-    bool operator!=(const Level& l) const;
-    bool operator<(const Level& l) const;
-    bool operator>(const Level& l) const;
+    bool operator==(const Level& o) const;
+    bool operator!=(const Level& o) const;
+    bool operator<(const Level& o) const;
+    bool operator>(const Level& o) const;
+    bool operator<=(const Level& o) const;
+    bool operator>=(const Level& o) const;
 
     /**
      * Generic comparison
@@ -568,18 +662,23 @@ struct Level
     /// Format to an output stream
     void to_stream(std::ostream& out, const char* undef="-") const;
 
+    /// Format to a string
+    std::string to_string(const char* undef="-") const;
+
     /**
      * Write the datetime to a CSV writer as 4 fields
      */
     void to_csv(CSVWriter& out) const;
 
-    /// Create a cloud special level. See @ref level_table.
+    /// Create a cloud special level
     static Level cloud(int ltype2=MISSING_INT, int l2=MISSING_INT);
 
     /// Print to an output stream
     int print(FILE* out, const char* undef="-", const char* end="\n") const;
 };
 
+std::ostream& operator<<(std::ostream& out, const Level& l);
+
 
 /**
  * Information on how a value has been sampled or computed with regards to time.
@@ -608,10 +707,12 @@ struct Trange
      */
     int compare(const Trange& t) const;
 
-    bool operator==(const Trange& tr) const;
-    bool operator!=(const Trange& tr) const;
-    bool operator<(const Trange& t) const;
-    bool operator>(const Trange& t) const;
+    bool operator==(const Trange& o) const;
+    bool operator!=(const Trange& o) const;
+    bool operator<(const Trange& o) const;
+    bool operator>(const Trange& o) const;
+    bool operator<=(const Trange& o) const;
+    bool operator>=(const Trange& o) const;
 
     /**
      * Return a string description of this time range
@@ -621,6 +722,9 @@ struct Trange
     /// Format to an output stream
     void to_stream(std::ostream& out, const char* undef="-") const;
 
+    /// Format to a string
+    std::string to_string(const char* undef="-") const;
+
     /**
      * Write the datetime to a CSV writer as 3 fields
      */
@@ -633,8 +737,216 @@ struct Trange
     int print(FILE* out, const char* undef="-", const char* end="\n") const;
 };
 
+std::ostream& operator<<(std::ostream& out, const Trange& l);
 
 
+/**
+ * A station identifier, that can be any string (including the empty string) or
+ * a missing value.
+ */
+class Ident
+{
+protected:
+    char* value = nullptr;
+
+public:
+    Ident() = default;
+    Ident(const char* value);
+    Ident(const std::string& value);
+    Ident(const Ident& o);
+    Ident(Ident&& o);
+    ~Ident();
+    Ident& operator=(const Ident& o);
+    Ident& operator=(Ident&& o);
+    Ident& operator=(const char* o);
+    Ident& operator=(const std::string& o);
+
+    /// Get the string value (might be nullptr in case of missing value)
+    const char* get() const { return value; }
+
+    /// Set to missing value
+    void clear();
+
+    int compare(const Ident& o) const;
+    int compare(const char* o) const;
+    int compare(const std::string& o) const;
+    template<typename T> bool operator==(const T& o) const { return compare(o) == 0; }
+    template<typename T> bool operator!=(const T& o) const { return compare(o) != 0; }
+    template<typename T> bool operator<(const T& o) const  { return compare(o) < 0; }
+    template<typename T> bool operator<=(const T& o) const { return compare(o) <= 0; }
+    template<typename T> bool operator>(const T& o) const  { return compare(o) > 0; }
+    template<typename T> bool operator>=(const T& o) const { return compare(o) >= 0; }
+
+    /// Check if the Ident is set to the missing value
+    bool is_missing() const;
+
+    operator const char*() const { return value; }
+    operator std::string() const;
+};
+
+std::ostream& operator<<(std::ostream&, const Ident&);
+
+
+/**
+ * Station information
+ */
+struct Station
+{
+    /// Report name for this station
+    std::string report;
+
+    /// Station coordinates
+    Coords coords;
+
+    /// Mobile station identifier
+    Ident ident;
+
+
+    Station() = default;
+
+    /// Return true if all the station fields are empty
+    bool is_missing() const;
+
+    bool operator==(const Station& o) const
+    {
+        return std::tie(report, coords, ident) == std::tie(o.report, o.coords, o.ident);
+    }
+    bool operator!=(const Station& o) const
+    {
+        return std::tie(report, coords, ident) != std::tie(o.report, o.coords, o.ident);
+    }
+    bool operator<(const Station& o) const
+    {
+        return std::tie(report, coords, ident) < std::tie(o.report, o.coords, o.ident);
+    }
+    bool operator<=(const Station& o) const
+    {
+        return std::tie(report, coords, ident) <= std::tie(o.report, o.coords, o.ident);
+    }
+    bool operator>(const Station& o) const
+    {
+        return std::tie(report, coords, ident) > std::tie(o.report, o.coords, o.ident);
+    }
+    bool operator>=(const Station& o) const
+    {
+        return std::tie(report, coords, ident) >= std::tie(o.report, o.coords, o.ident);
+    }
+
+    /**
+     * Print the Station to a FILE*.
+     *
+     * @param out  The output stream
+     * @param end  String to print after the Station
+     */
+    int print(FILE* out, const char* end="\n") const;
+
+    /// Format to a string
+    std::string to_string(const char* undef="-") const;
+};
+
+std::ostream& operator<<(std::ostream&, const Station&);
+
+
+struct DBStation : public Station
+{
+    /**
+     * Database ID of the station.
+     *
+     * It will be filled when the Station is inserted on the database.
+     */
+    int id = MISSING_INT;
+
+
+    DBStation() = default;
+
+    /// Return true if all the station fields are empty
+    bool is_missing() const;
+
+    bool operator==(const DBStation& o) const
+    {
+        return std::tie(id, report, coords, ident) == std::tie(o.id, o.report, o.coords, o.ident);
+    }
+    bool operator!=(const DBStation& o) const
+    {
+        return std::tie(id, report, coords, ident) != std::tie(o.id, o.report, o.coords, o.ident);
+    }
+    bool operator<(const DBStation& o) const
+    {
+        return std::tie(id, report, coords, ident) < std::tie(o.id, o.report, o.coords, o.ident);
+    }
+    bool operator<=(const DBStation& o) const
+    {
+        return std::tie(id, report, coords, ident) <= std::tie(o.id, o.report, o.coords, o.ident);
+    }
+    bool operator>(const DBStation& o) const
+    {
+        return std::tie(id, report, coords, ident) > std::tie(o.id, o.report, o.coords, o.ident);
+    }
+    bool operator>=(const DBStation& o) const
+    {
+        return std::tie(id, report, coords, ident) >= std::tie(o.id, o.report, o.coords, o.ident);
+    }
+
+    /**
+     * Print the Station to a FILE*.
+     *
+     * @param out  The output stream
+     * @param end  String to print after the Station
+     */
+    int print(FILE* out, const char* end="\n") const;
+
+    /// Format to a string
+    std::string to_string(const char* undef="-") const;
+};
+
+std::ostream& operator<<(std::ostream&, const DBStation&);
+
+}
+
+namespace std {
+
+template<> struct hash<dballe::Level>
+{
+    typedef dballe::Level argument_type;
+    typedef size_t result_type;
+    result_type operator()(argument_type const& o) const noexcept;
+};
+
+template<> struct hash<dballe::Trange>
+{
+    typedef dballe::Trange argument_type;
+    typedef size_t result_type;
+    result_type operator()(argument_type const& o) const noexcept;
+};
+
+template<> struct hash<dballe::Coords>
+{
+    typedef dballe::Coords argument_type;
+    typedef size_t result_type;
+    result_type operator()(argument_type const& o) const noexcept;
+};
+
+template<> struct hash<dballe::Ident>
+{
+    typedef dballe::Ident argument_type;
+    typedef size_t result_type;
+    result_type operator()(argument_type const& o) const noexcept;
+};
+
+template<> struct hash<dballe::Station>
+{
+    typedef dballe::Station argument_type;
+    typedef size_t result_type;
+    result_type operator()(argument_type const& o) const noexcept;
+};
+
+template<> struct hash<dballe::DBStation>
+{
+    typedef dballe::DBStation argument_type;
+    typedef size_t result_type;
+    result_type operator()(argument_type const& o) const noexcept;
+};
+
 }
 
 #endif
