@@ -17,22 +17,11 @@ class Tests : public TestCase
         add_method("resolve", []() {
             wassert(actual(resolve_varcode("B12101")) == WR_VAR(0, 12, 101));
             wassert(actual(resolve_varcode("t")) == WR_VAR(0, 12, 101));
-            try {
-                resolve_varcode("B121");
-                wassert(throw TestFailed("an exception should be thrown"));
-            } catch (TestFailed) {
-                throw;
-            } catch (std::exception& e) {
-                wassert(actual(e.what()).contains("cannot parse"));
-            }
-            try {
-                resolve_varcode("B1210A");
-                wassert(throw TestFailed("an exception should be thrown"));
-            } catch (TestFailed) {
-                throw;
-            } catch (std::exception& e) {
-                wassert(actual(e.what()).contains("cannot parse"));
-            }
+            auto e = wassert_throws(error_notfound, resolve_varcode("B121"));
+            wassert(actual(e.what()).contains("cannot parse"));
+
+            auto e1 = wassert_throws(error_notfound, resolve_varcode("B1210A"));
+            wassert(actual(e1.what()).contains("cannot parse"));
         });
         add_method("varinfo", []() {
             Varinfo i = varinfo(WR_VAR(0, 12, 101));
