@@ -116,11 +116,8 @@
       ier = idba_arrivederci(dbahandle)
       call ensure_no_error("arrivederci")
 
-!     If we made it so far, exit with no error
-      print*,"check_messages: all tests succeed."
-
       call exit (0)
-    
+
       end program
 
       include "check-utils.h"
