@@ -84,12 +84,9 @@
 
       ierr = idba_arrivederci(dbahandle)
       call ensure_no_error("arrivederci")
-  
-!     If we made it so far, exit with no error
-      print*,"check_missing: all tests succeed."
 
       call exit (0)
-    
+
       end program
 
       include "check-utils.h"
