@@ -5,10 +5,9 @@
 ! *****************************************
 
       use dbtest
+      use dballef
 
-      include "dballef.h" 
-      
-      integer :: handle,idbhandle,handle_err
+      integer :: handle,idbhandle,handle_err, ierr
       !data var/ "B22070", "B22074", "B22001", "B22071", "B22042"/
       !integer debug
       !data debug/1/
@@ -19,41 +18,41 @@
       call dbinit(idbhandle)
 
 !     Open a session
-      call idba_preparati(idbhandle,handle,"write","write","write")
+      ierr = idba_preparati(idbhandle,handle,"write","write","write")
       call ensure_no_error("preparati")
 
 !     Clear the database
-      call idba_scopa(handle, "") 
+      ierr = idba_scopa(handle, "") 
       call ensure_no_error("scopa")
 
 !     Insert some data with a 0.0D0 value
-      call idba_unsetall (handle)
+      ierr = idba_unsetall (handle)
 
-      call idba_set (handle,"lat",44.2)
+      ierr = idba_set (handle,"lat",44.2)
       call ensure_no_error("set lat")
-      call idba_set (handle,"lon",11.5)
+      ierr = idba_set (handle,"lon",11.5)
       call ensure_no_error("set lon")
-      call idba_set (handle,"mobile",0)
+      ierr = idba_set (handle,"mobile",0)
       call ensure_no_error("set mobile")
-      call idba_set (handle,"rep_memo","synop")
+      ierr = idba_set (handle,"rep_memo","synop")
       call ensure_no_error("set rep_memo")
-      call idba_setlevel (handle,1,0,0,0)
+      ierr = idba_setlevel (handle,1,0,0,0)
       call ensure_no_error("set level")
-      call idba_settimerange (handle,0,0,0)
+      ierr = idba_settimerange (handle,0,0,0)
       call ensure_no_error("set timerange")
-      call idba_setdate (handle,2007,12,11,12,30,00)
+      ierr = idba_setdate (handle,2007,12,11,12,30,00)
       call ensure_no_error("set date")
      
-      call idba_set(handle,"B22070",0.0D0)
+      ierr = idba_set(handle,"B22070",0.0D0)
       call ensure_no_error("set B22070 to 0.0D0")
   
-      call idba_prendilo (handle)
+      ierr = idba_prendilo (handle)
       call ensure_no_error("prendilo")
      
-      call idba_fatto(handle)
+      ierr = idba_fatto(handle)
       call ensure_no_error("fatto")
 
-      call idba_arrivederci(idbhandle)
+      ierr = idba_arrivederci(idbhandle)
       call ensure_no_error("arrivederci")
   
       call exit (0)
