@@ -8,7 +8,7 @@ integer :: error_handle
 
 ierr = idba_error_set_callback(0, C_FUNLOC(idba_default_error_handler), 42, error_handle)
 
-ierr = idba_presentati(idbhandle1,"sqlite:tmp.sqlite")
+ierr = idba_presentati(idbhandle1,"sqlite:tmp.sqlite?wipe=1")
 ierr = idba_preparati (idbhandle1,handle1,"write","write","write")
 ierr = idba_scopa (handle1, "")
 ierr = idba_fatto(handle1)
