@@ -21,7 +21,6 @@
 #define FDBA_HANDLES_H
 
 #include <wreport/error.h>
-#include <cassert>
 
 namespace dballe {
 struct DB;
@@ -29,53 +28,6 @@ struct DB;
 namespace fortran {
 struct API;
 
-#if 0
-#define FDBA_HANDLE_END_DECL(name) }; \
-	extern struct fdba_handle_##name _##name[]; \
-	void fdba_handle_init_##name(); \
-	dba_err fdba_handle_alloc_##name(int* res); \
-	void fdba_handle_release_##name(int hnd);
-
-
-#define FDBA_HANDLE_BODY(name, size, desc) \
-	struct fdba_handle_##name _##name[size]; \
-	static int _##name##_last = 0; \
-	/* Initialize the handle storage */ \
-	void fdba_handle_init_##name() { \
-		int i; \
-		for (i = 0; i < size; i++) \
-			_##name[i].used = 0; \
-	} \
-	/* Allocate a new handle */ \
-	dba_err fdba_handle_alloc_##name(int* res) { \
-		if (_##name##_last < size) { \
-			*res = _##name##_last++; \
-		} else { \
-			int i, found = 0; \
-			for (i = 0; i < size && !found; i++) \
-				if (_##name[i].used == 0) { \
-					*res = i; \
-					found = 1; \
-				} \
-			if (!found) \
-				return dba_error_handles("No more handles for " desc ". The maximum limit is %d: to increase it, recompile dballe setting " #size " to a higher value", size); \
-		} \
-		/* Setup the new handle */ \
-		_##name[*res].used = 1; \
-		return dba_error_ok(); \
-	} \
-	/* Release a handle */ \
-	void fdba_handle_release_##name(int hnd) { \
-		assert(hnd < size); \
-		assert(_##name[hnd].used == 1); \
-		_##name[hnd].used = 0; \
-		if (hnd == _##name##_last - 1) \
-			_##name##_last--; \
-	}
-
-#define FDBA_HANDLE(name, hnd) (_##name[hnd])
-#endif
-
 struct HBase
 {
 	bool used;
@@ -104,11 +56,12 @@ struct Handler
 		in_use = 0;
 	}
 
-	T& get(int id)
-	{
-		assert(records[id].used);
-		return records[id];
-	}
+    T& get(int id)
+    {
+        if (!records[id].used)
+            wreport::error_handles::throwf("Handle %d used after it has been closed, or when it has never been opened", id);
+        return records[id];
+    }
 
 	int request()
 	{
@@ -122,16 +75,13 @@ struct Handler
 		return next;
 	}
 
-	void release(int h)
-	{
-		assert(in_use);
-		assert(records[h].used);
-		if (records[h].used)
-		{
-			records[h].stop();
-			--in_use;
-		}
-	}
+    void release(int h)
+    {
+        if (!in_use) wreport::error_handles::throwf("Attempted to close handle %d when no handles are in use", h);
+        if (!records[h].used) wreport::error_handles::throwf("Attempted to close handle %d which was not open", h);
+        records[h].stop();
+        --in_use;
+    }
 };
 
 /// Initialise error handlers
