@@ -1,5 +1,5 @@
 from __future__ import absolute_import
-import wreport
 from wreport import Var, Varinfo, Vartable
 import _dballe
 from _dballe import *
+from _dballe import __version__
