@@ -1,12 +1,8 @@
-#!/usr/bin/python
-# coding: utf-8
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
+#!/usr/bin/python3
 import dballe
 import unittest
 
+
 class Describe(unittest.TestCase):
     def testLevel(self):
         self.assertIn("surface", dballe.describe_level(1))
