@@ -1,11 +1,9 @@
 #include <dballe/cmdline/dbadb.h>
 #include <dballe/cmdline/processor.h>
 #include <dballe/cmdline/cmdline.h>
-#include <dballe/record.h>
 #include <dballe/file.h>
 #include <dballe/message.h>
 #include <dballe/msg/msg.h>
-#include <dballe/msg/codec.h>
 #include <dballe/db/db.h>
 #include <wreport/error.h>
 #include <wreport/utils/string.h>
@@ -26,6 +24,7 @@ const char* op_report = "";
 const char* op_url = "";
 const char* op_user = "";
 const char* op_pass = "";
+const char* op_varlist = "";
 int op_wipe_first = 0;
 int op_dump = 0;
 int op_overwrite = 0;
@@ -61,7 +60,7 @@ struct poptOption dbTable[] = {
     POPT_TABLEEND
 };
 
-static unique_ptr<DB> connect()
+static std::shared_ptr<db::DB> connect()
 {
     const char* chosen_url;
 
@@ -75,7 +74,8 @@ static unique_ptr<DB> connect()
         chosen_url = op_url;
 
     /* If url looks like a url, treat it accordingly */
-    unique_ptr<DB> db = DB::connect_from_url(chosen_url);
+    auto options = DBConnectOptions::create(chosen_url);
+    auto db = dynamic_pointer_cast<db::DB>(DB::connect(*options));
 
     // Wipe database if requested
     if (op_wipe_first)
@@ -119,7 +119,7 @@ struct DumpCmd : public DatabaseCmd
         core::Query query;
         dba_cmdline_get_query(optCon, query);
 
-        unique_ptr<DB> db = connect();
+        auto db = connect();
         Dbadb dbadb(*db);
 
         return dbadb.do_dump(query, stdout);
@@ -148,7 +148,7 @@ struct StationsCmd : public DatabaseCmd
         core::Query query;
         dba_cmdline_get_query(optCon, query);
 
-        unique_ptr<DB> db = connect();
+        auto db = connect();
         Dbadb dbadb(*db);
 
         return dbadb.do_stations(query, stdout);
@@ -186,14 +186,14 @@ struct WipeCmd : public DatabaseCmd
 
         {
             // Connect first using the current format, and remove all tables.
-            unique_ptr<DB> db = connect();
+            auto db = connect();
             db->disappear();
         }
 
         if (!op_wipe_disappear)
         {
             // Recreate tables
-            unique_ptr<DB> db = connect();
+            auto db = connect();
             db->reset(fname);
         }
         return 0;
@@ -216,7 +216,7 @@ struct CleanupCmd : public DatabaseCmd
 
     int main(poptContext optCon)
     {
-        unique_ptr<DB> db = connect();
+        auto db = connect();
         db->vacuum();
         return 0;
     }
@@ -241,13 +241,16 @@ struct RepinfoCmd : public DatabaseCmd
         /* Throw away the command name */
         poptGetArg(optCon);
 
-        unique_ptr<DB> db = connect();
+        auto db = connect();
 
         /* Get the optional name of the repinfo file.  If missing, the default will be used */
         const char* fname = poptGetArg(optCon);
 
+        auto tr = dynamic_pointer_cast<db::Transaction>(db->transaction());
         int added, deleted, updated;
-        db->update_repinfo(fname, &added, &deleted, &updated);
+        tr->update_repinfo(fname, &added, &deleted, &updated);
+        tr->commit();
+
         printf("Update completed: %d added, %d deleted, %d updated.\n", added, deleted, updated);
         return 0;
     }
@@ -266,7 +269,7 @@ struct ImportCmd : public DatabaseCmd
     {
         DatabaseCmd::add_to_optable(opts);
         opts.push_back({ "type", 't', POPT_ARG_STRING, &readeropts.input_type, 0,
-            "format of the input data ('bufr', 'crex', 'aof', 'csv', 'json')", "type" });
+            "format of the input data ('bufr', 'crex', 'csv', 'json')", "type" });
         opts.push_back({ "rejected", 0, POPT_ARG_STRING, &readeropts.fail_file_name, 0,
             "write unprocessed data to this file", "fname" });
         opts.push_back({ "overwrite", 'f', POPT_ARG_NONE, &op_overwrite, 0,
@@ -282,6 +285,8 @@ struct ImportCmd : public DatabaseCmd
             "merge pseudoana extra values with the ones already existing in the database", 0 });
         opts.push_back({ "precise", 0, 0, &op_precise_import, 0,
             "import messages using precise contexts instead of standard ones", 0 });
+        opts.push_back({ "varlist", 0, POPT_ARG_STRING, &op_varlist, 0,
+            "only import variables with the given varcode(s)", "varlist" });
         opts.push_back({ NULL, 0, POPT_ARG_INCLUDE_TABLE, &grepTable, 0,
             "Options used to filter messages", 0 });
     }
@@ -291,6 +296,7 @@ struct ImportCmd : public DatabaseCmd
         // Throw away the command name
         poptGetArg(optCon);
         cmdline::Reader reader(readeropts);
+        reader.verbose = op_verbose;
 
         // Configure the reader
         core::Query query;
@@ -299,24 +305,25 @@ struct ImportCmd : public DatabaseCmd
         reader.import_opts.simplified = !op_precise_import;
 
         // Configure the importer
-        int import_flags = 0;
+        auto opts = DBImportOptions::create();
         if (op_overwrite)
-            import_flags |= DBA_IMPORT_OVERWRITE;
+            opts->overwrite = true;
         if (op_fast)
             setenv("DBA_INSECURE_SQLITE", "true", true);
         if (!op_no_attrs)
-            import_flags |= DBA_IMPORT_ATTRS;
+            opts->import_attributes = true;
         if (op_full_pseudoana)
-            import_flags |= DBA_IMPORT_FULL_PSEUDOANA;
+            opts->update_station = true;
+        if (op_varlist[0])
+            resolve_varlist(op_varlist, [&](wreport::Varcode code) { opts->varlist.push_back(code); });
 
-        unique_ptr<DB> db = connect();
+        auto db = connect();
 
-        const char* forced_repmemo = NULL;
         if (strcmp(op_report, "") != 0)
-            forced_repmemo = op_report;
+            opts->report = op_report;
 
         Dbadb dbadb(*db);
-        return dbadb.do_import(get_filenames(optCon), reader, import_flags, forced_repmemo);
+        return dbadb.do_import(get_filenames(optCon), reader, *opts);
     }
 };
 
@@ -325,12 +332,14 @@ struct ExportCmd : public DatabaseCmd
     ExportCmd()
     {
         names.push_back("export");
-        usage = "export [options] rep_memo [queryparm1=val1 [queryparm2=val2 [...]]]";
+        usage = "export [options] [queryparm1=val1 [queryparm2=val2 [...]]]";
         desc = "Export data from the database";
         longdesc = "Query parameters are the same of the Fortran API. "
             "Please see the section \"Input and output parameters -- For data "
             "related action routines\" of the Fortran API documentation for a "
-            "complete list.";
+            "complete list.\n\n"
+            "The database is specified with --url or with the DBA_DB"
+            " environment variable";
     }
 
     void add_to_optable(std::vector<poptOption>& opts) const override
@@ -339,7 +348,7 @@ struct ExportCmd : public DatabaseCmd
         opts.push_back({ "report", 'r', POPT_ARG_STRING, &op_report, 0,
             "force exported data to be of this type of report", "rep" });
         opts.push_back({ "dest", 'd', POPT_ARG_STRING, &op_output_type, 0,
-            "format of the data in output ('bufr', 'crex', 'aof')", "type" });
+            "format of the data in output ('bufr', 'crex')", "type" });
         opts.push_back({ "template", 't', POPT_ARG_STRING, &op_output_template, 0,
             "template of the data in output (autoselect if not specified, 'list' gives a list)", "name" });
         opts.push_back({ "dump", 0, POPT_ARG_NONE, &op_dump, 0,
@@ -361,7 +370,7 @@ struct ExportCmd : public DatabaseCmd
         core::Query query;
         dba_cmdline_get_query(optCon, query);
 
-        unique_ptr<DB> db = connect();
+        auto db = connect();
         Dbadb dbadb(*db);
 
         const char* forced_repmemo = NULL;
@@ -372,8 +381,8 @@ struct ExportCmd : public DatabaseCmd
         {
             return dbadb.do_export_dump(query, stdout);
         } else {
-            File::Encoding type = File::parse_encoding(op_output_type);
-            unique_ptr<File> file = File::create(type, stdout, false, "w");
+            Encoding type = File::parse_encoding(op_output_type);
+            auto file = File::create(type, stdout, false, "w");
             return dbadb.do_export(query, *file, op_output_template, forced_repmemo);
         }
     }
@@ -404,9 +413,9 @@ struct DeleteCmd : public DatabaseCmd
         core::Query query;
         dba_cmdline_get_query(optCon, query);
 
-        unique_ptr<DB> db = connect();
+        auto db = connect();
         // TODO: check that there is something
-        db->remove(query);
+        db->remove_data(query);
         return 0;
     }
 };
@@ -425,9 +434,9 @@ struct InfoCmd : public DatabaseCmd
         // Throw away the command name
         poptGetArg(optCon);
 
-        unique_ptr<DB> db = connect();
+        auto db = connect();
 
-        string default_format = db::format_format(DB::get_default_format());
+        string default_format = db::format_format(db::DB::get_default_format());
         fprintf(stdout, "Default format for new DBs: %s\n", default_format.c_str());
         db->print_info(stdout);
 
@@ -443,7 +452,7 @@ int main (int argc, const char* argv[])
     dbadb.desc = "Manage the DB-ALLe database";
     dbadb.longdesc =
         "It allows to initialise the database, dump its contents and import and export data "
-        "using BUFR, CREX or AOF encoding";
+        "using BUFR, or CREX encoding";
 
     dbadb.add_subcommand(new DumpCmd);
     dbadb.add_subcommand(new StationsCmd);
